import cElementTree as ElementTree

class XmlListConfig(list):
    def __init__(self, aList):
        for element in aList:
            if element:
                # treat like dict
                if len(element) == 1 or element[0].tag != element[1].tag:
                    self.append(XmlDictConfig(element))
                # treat like list
                elif element[0].tag == element[1].tag:
                    self.append(XmlListConfig(element))
            elif element.text:
                text = element.text.strip()
                if text:
                    self.append(text)


class XmlDictConfig(dict):
    '''
    Example usage:

    >>> tree = ElementTree.parse('your_file.xml')
    >>> root = tree.getroot()
    >>> xmldict = XmlDictConfig(root)

    Or, if you want to use an XML string:

    >>> root = ElementTree.XML(xml_string)
    >>> xmldict = XmlDictConfig(root)

    And then use xmldict for what it is... a dict.
    '''
    def __init__(self, parent_element):
        if list(parent_element.items()):
            self.update(dict(list(parent_element.items())))
        for element in parent_element:
            if element:
                # treat like dict - we assume that if the first two tags
                # in a series are different, then they are all different.
                if len(element) == 1 or element[0].tag != element[1].tag:
                    aDict = XmlDictConfig(element)
                # treat like list - we assume that if the first two tags
                # in a series are the same, then the rest are the same.
                else:
                    # here, we put the list in dictionary; the key is the
                    # tag name the list elements all share in common, and
                    # the value is the list itself 
                    aDict = {element[0].tag: XmlListConfig(element)}
                # if the tag has attributes, add those to the dict
                if list(element.items()):
                    aDict.update(dict(list(element.items())))
                self.update({element.tag: aDict})
            # this assumes that if you've got an attribute in a tag,
            # you won't be having any text. This may or may not be a 
            # good idea -- time will tell. It works for the way we are
            # currently doing XML configuration files...
            elif list(element.items()):
                self.update({element.tag: dict(list(element.items()))})
            # finally, if there are no child tags and no attributes, extract
            # the text
            else:
                self.update({element.tag: element.text})


import sys
from xml import sax
from .textnormalize import text_normalize_filter
import pdb

#Subclass from ContentHandler in order to gain default behaviors
class label_dict_handler(sax.ContentHandler):
    #Define constants for important states
    CAPTURE_KEY = 1
    CAPTURE_LABEL_ITEM = 2
    CAPTURE_ADDRESS_ITEM = 3

    def __init__(self):
        self.label_dict = {}
        #Track the item being constructed in the current dictionary
        self._item_to_create = None
        self._state = None
        return

    def startElement(self, name, attributes):
        if name == "label":
            self._curr_label = {}
        if name == "address":
            self._address = {}
        if name == "name":
            self._state = self.CAPTURE_KEY
        if name == "quote":
            self._item_to_create = name
            self._state = self.CAPTURE_LABEL_ITEM
        if name in ["street", "city", "state"]:
            self._item_to_create = name
            self._state = self.CAPTURE_ADDRESS_ITEM
        return

    def endElement(self, name):
        if name == "address":
            self._curr_label["address"] = self._address
        if name in ["quote", "name", "street", "city", "state"]:
            self._state = None
        return

    def characters(self, text):
        if self._state == self.CAPTURE_KEY:
            self.label_dict[text] = self._curr_label
        curr_dict = None
        if self._state == self.CAPTURE_ADDRESS_ITEM:
            curr_dict = self._address
        if self._state == self.CAPTURE_LABEL_ITEM:
            curr_dict = self._curr_label
        print(repr(text), curr_dict)
        if curr_dict is not None:
            if self._item_to_create in curr_dict:
                curr_dict[self._item_to_create] += text
            else:
                curr_dict[self._item_to_create] = text
        return

if __name__ == "__main__":
    parser = sax.make_parser()
    downstream_handler = label_dict_handler()
    #upstream: the parser; downstream: the next handler in the chain
    filter_handler = text_normalize_filter(parser, downstream_handler)
    #XMLFilterBase is designed so that the filter takes on much of the
    #interface of the parser itself, including the "parse" method
    filter_handler.parse(sys.argv[1])
    label_dict = downstream_handler.label_dict 
    pdb.set_trace()


#!/usr/bin/python
###########################################################################################
#   ultrasignupresults - manage race results data from ultrasignup
#
#   Date        Author      Reason
#   ----        ------      ------
#   11/13/13    Lou King    Create
#
#   Copyright 2013 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
ultrasignupresults - manage race results data from ultrasignup
===================================================================

Usage::
    ultrasignupresults.py [-h] [-v] [-b BEGINDATE] [-e ENDDATE]
                                     searchfile outfile
    
        collect race results from ultrasignup
    
        searchfile must have at least the following headings:
    
            * GivenName - first name
            * FamilyName - last name
            * Gender - Male or Female (or M or F)
            * DOB - date of birth in yyyy-mm-dd format
            * City - city of residence [optional]
            * State - state of residence [optional]
    
    
    positional arguments:
      searchfile            file with names, genders and birth dates of athletes
                            to search for
      outfile               output file contains race results
    
    optional arguments:
      -h, --help            show this help message and exit
      -v, --version         show program's version number and exit
      -b BEGINDATE, --begindate BEGINDATE
                            choose races between begindate and enddate, yyyy-mm-dd
      -e ENDDATE, --enddate ENDDATE
                            choose races between begindate and enddate, yyyy-mm-dd
                        
'''

# standard
import argparse
import csv
import datetime
import time

# pypi
#from IPython.core.debugger import Tracer; debug_here = Tracer()

# github

# other

# home grown
from loutilities import timeu
from loutilities import csvu
from runningclub import agegrade
from running.running import version, ultrasignup

# see http://api.ultrasignup.com/Enums/RaceCategories
ag = agegrade.AgeGrade()
class invalidParameter(Exception): pass

ftime = timeu.asctime('%Y-%m-%d')

#----------------------------------------------------------------------
def collect(searchfile,outfile,begindate,enddate):
#----------------------------------------------------------------------
    '''
    collect race results from ultrasignup
    
    :param searchfile: path to file containing names, genders, birth dates to search for
    :param outfile: output file path
    :param begindate: epoch time - choose races between begindate and enddate
    :param enddate: epoch time - choose races between begindate and enddate
    '''
    
    # open files
    _IN = open(searchfile,'r',newline='')
    IN = csv.DictReader(_IN)
    _OUT = open(outfile,'w',newline='')
    OUT = csv.DictWriter(_OUT,UltraSignupResultFile.filehdr)
    OUT.writeheader()

    # common fields between input and output
    commonfields = 'GivenName,FamilyName,DOB,Gender'.split(',')

    # create ultrasignup access
    ultra = ultrasignup.UltraSignup(debug=True)

    # reset begindate to beginning of day, enddate to end of day
    dt_begindate = timeu.epoch2dt(begindate)
    adj_begindate = datetime.datetime(dt_begindate.year,dt_begindate.month,dt_begindate.day,0,0,0)
    begindate = timeu.dt2epoch(adj_begindate)
    dt_enddate = timeu.epoch2dt(enddate)
    adj_enddate = datetime.datetime(dt_enddate.year,dt_enddate.month,dt_enddate.day,23,59,59)
    enddate = timeu.dt2epoch(adj_enddate)
    
    # get today's date for high level age filter
    start = time.time()
    today = timeu.epoch2dt(start)
    
    # loop through runners in the input file
    for runner in IN:
        fname,lname = runner['GivenName'],runner['FamilyName']
        e_dob = ftime.asc2epoch(runner['DOB'])
        dt_dob = ftime.asc2dt(runner['DOB'])
        gender = runner['Gender'][0]
        
        ## skip getting results if participant too young
        #todayage = timeu.age(today,dt_dob)
        #if todayage < 14: continue
        
        # get results for this athlete
        results = ultra.listresults(fname,lname)
        
        # loop through each result
        for result in results:
            e_racedate = ftime.asc2epoch(result.racedate)
            
            # skip result if outside the desired time window
            if e_racedate < begindate or e_racedate > enddate: continue
            
            # skip result if runner's age doesn't match the age within the result
            dt_racedate = timeu.epoch2dt(e_racedate)
            racedateage = timeu.age(dt_racedate,dt_dob)
            if result.age != racedateage: continue
            
            # skip result if runner's gender doesn't match gender within the result
            resultgen = result.gender
            if resultgen != runner['Gender'][0]: continue
            
            # create output record and copy common fields
            outrec = {}
            for field in commonfields:
                outrec[field] = runner[field]
                
            # fill in output record fields from runner, result
            # combine name, get age
            outrec['name'] = '{} {}'.format(runner['GivenName'],runner['FamilyName'])
            outrec['age'] = result.age

            # race name, location; convert from unicode if necessary
            racename = result.racename
            outrec['race'] = racename
            outrec['date'] = ftime.epoch2asc(e_racedate)
            outrec['loc'] = '{}, {}'.format(result.racecity, result.racestate)
            
            # distance, category, time
            distmiles = result.distmiles
            distkm = result.distkm
            if distkm is None or distkm < 0.050: continue # should already be filtered within ultrasignup, but just in case

            outrec['miles'] = distmiles
            outrec['km'] = distkm
            resulttime = result.racetime

            # int resulttime means DNF, most likely -- skip this result
            if isinstance(resulttime, int): continue
            
            # strange case of TicksString = ':00'
            if resulttime[0] == ':':
                resulttime = '0'+resulttime
            while resulttime.count(':') < 2:
                resulttime = '0:'+resulttime
            outrec['time'] = resulttime

            # just leave out age grade if exception occurs
            try:
                agpercent,agresult,agfactor = ag.agegrade(racedateage,gender,distmiles,timeu.timesecs(resulttime))
                outrec['ag'] = agpercent
                if agpercent < 15 or agpercent >= 100: continue # skip obvious outliers
            except:
                pass

            OUT.writerow(outrec)
        
    _OUT.close()
    _IN.close()
    
    finish = time.time()
    print('number of URLs retrieved = {}'.format(ultra.geturlcount()))
    print('elapsed time (min) = {}'.format((finish-start)/60))
    
########################################################################
class UltraSignupFileResult():
########################################################################
    '''
    holds result from ultrasignup file
    
    :param firstname: first name
    :param lastname: last name
    :param name: firstname lastname
    :param dob: date of birth, datetime
    :param gender: M or F
    :param race: name of race
    :param date: date of race, datetime
    :param loc: location of race
    :param age: age on race day
    :param miles: race distance, miles
    :param km: race distance, kilometers
    :param time: race time, seconds
    :param ag: age grade percentage
    '''
    attrs = 'firstname,lastname,name,dob,gender,race,date,loc,age,miles,km,time,ag'.split(',')
    
    #----------------------------------------------------------------------
    def __init__(self,firstname=None,lastname=None,name=None,dob=None,gender=None,race=None,date=None,loc=None,age=None,miles=None,km=None,time=None,ag=None):
    #----------------------------------------------------------------------
        self.firstname = firstname
        self.lastname = lastname
        self.name = name
        self.dob = dob
        self.gender = gender
        self.race = race
        self.date = date
        self.loc = loc
        self.age = age
        self.miles = miles
        self.km = km
        self.time = time
        self.ag = ag
        
    #----------------------------------------------------------------------
    def __repr__(self):
    #----------------------------------------------------------------------
        reprval = '{}('.format(self.__class__)
        for attr in self.attrs:
            val = getattr(self,attr)
            if attr in ['dob','date']:
                val = ftime.dt2asc(val)
            reprval += '{}={},'.format(attr,val)
        reprval = reprval[:-1]
        reprval += ')'
        return reprval
    
    #----------------------------------------------------------------------
    def set(self,attrvals):
    #----------------------------------------------------------------------
        '''
        set attributes based on list of attr,val pairs
        
        :param attrvals: [(attr,val),...]
        '''
        
        for attr,inval in attrvals:
            val = csvu.str2num(inval)
            
            setattr(self,attr,val)

########################################################################
class UltraSignupResultFile():
########################################################################
    '''
    represents file of ultrasignup results collected from ultrasignup
    
    TODO:: add write methods, and update :func:`collect` to use :class:`UltraSignupFileResult` class
    '''
    filehdr = 'GivenName,FamilyName,name,DOB,Gender,race,date,loc,age,miles,km,time,ag'.split(',')
    # UltraSignupResultFile.filehdr needs to associate 1:1 with UltraSignupFileResult.attrs
    hdrtransform = dict(list(zip(filehdr,UltraSignupFileResult.attrs)))

    resultdates = 'dob,date'.split(',')

    #----------------------------------------------------------------------
    def __init__(self,filename):
    #----------------------------------------------------------------------
        self.filename = filename
        
    #----------------------------------------------------------------------
    def open(self,mode='r'):
    #----------------------------------------------------------------------
        '''
        open ultrasignup result file
        
        :param mode: 'r' or 'w' -- TODO: support 'w'
        '''
        if mode[0] not in 'r':
            raise invalidParameter('mode {} not currently supported'.format(mode))
    
        self._fh = open(self.filename,mode,newline='')
        if mode[0] == 'r':
            self._csv = csv.DictReader(self._fh)
        else:
            pass
        
    #----------------------------------------------------------------------
    def close(self):
    #----------------------------------------------------------------------
        '''
        close ultrasignup result file
        '''
        if hasattr(self,'_fh'):
            self._fh.close()
            delattr(self,'_fh')
            delattr(self,'_csv')
        
    #----------------------------------------------------------------------
    def __next__(self):
    #----------------------------------------------------------------------
        '''
        get next :class:`UltraSignupFileResult`
        
        :rtype: :class:`UltraSignupFileResult`, or None when end of file reached
        '''
        try:
            fresult = next(self._csv)
            
        except StopIteration:
            return None
        
        aresultargs = {}
        for fattr in self.hdrtransform:
            aattr = self.hdrtransform[fattr]
            
            # special handling for gender
            if aattr == 'gender':
                aresultargs[aattr] = fresult[fattr][0]
                
            # special handling for dates
            elif aattr in self.resultdates:
                aresultargs[aattr] = ftime.asc2dt(fresult[fattr])
                
            else:
                # convert numbers
                aresultargs[aattr] = csvu.str2num(fresult[fattr])
                
        return UltraSignupFileResult(**aresultargs)
    
#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    collect race results from ultrasignup
    
    searchfile must have at least the following headings:
    
        * GivenName - first name
        * FamilyName - last name
        * Gender - Male or Female (or M or F)
        * DOB - date of birth in yyyy-mm-dd format
        * City - city of residence [optional]
        * State - state of residence [optional]
    '''
    
    parser = argparse.ArgumentParser(description=descr, formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running', version.__version__))
    parser.add_argument('searchfile', help="file with names, genders and birth dates of athletes to search for")
    parser.add_argument('outfile', help="output file contains race results")
    parser.add_argument('-b','--begindate', help="choose races between begindate and enddate, yyyy-mm-dd",default=None)
    parser.add_argument('-e','--enddate', help="choose races between begindate and enddate, yyyy-mm-dd",default=None)
    args = parser.parse_args()

    searchfile = args.searchfile
    outfile = args.outfile

    argtime = timeu.asctime('%Y-%m-%d')
    if args.begindate:
        begindate = argtime.asc2epoch(args.begindate)
    else:
        begindate = argtime.asc2epoch('1970-01-01')
    if args.enddate:
        enddate = argtime.asc2epoch(args.enddate)
    else:
        enddate = argtime.asc2epoch('2030-12-31')
        
    # collect all the data
    collect(searchfile,outfile,begindate,enddate)
        
########################################################################
#	__main__
########################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
# ##########################################################################################
#	parserun.py
#		parse run from garmin training center
#
#	Date		Author		Reason
#	----		------		------
#	10/25/09	Lou King	Create
#
# ##########################################################################################

# standard
import optparse
import datetime
import re
import os
import math

# home grown
from running.running import xmldict

MHR = 204				# Lou's MHR.  Change for yours
METERPMILE = 1609.3439941
EPSILON = METERPMILE - 1600 + 1	# make sure 1600 ~= mile

# ##########################################################################################
def timestr(
	tsm):
#		return time in seconds per mile
# ##########################################################################################
	
	# parse out the integer and microsecond portion.  Add 1/2 second for rounding
	tsec = int(tsm)
	tusec = ((tsm-tsec)*1E6)+.5E6
	
	# add the duration in seconds, microseconds to epoch time of 0, allowing reformat using strftime function
	d = datetime.timedelta (0, tsec, tusec)
	dt1 = datetime.datetime(2009,1,1,0,0,0)
	dt2 = dt1 + d
	
	# if it's greater than an hour, then show the hours, else just show minutes:seconds
	if (tsec+(tusec/1E6) > 60*60):
		fmt = "%H:%M:%S" 
	else:
		fmt = "%M:%S"
	s = dt2.strftime(fmt)
	
	# get rid of leading 0, if present
	s = re.sub('^0','',s)
	
	return s;

# ##########################################################################################
def pace (
#		return pace in seconds per mile
	ts, 
#		time in seconds
	d):
#		distance in meters
# ##########################################################################################

	# calculate time in seconds per mile
	tsm = ts * (METERPMILE / d)
	
	return timestr(tsm)

# ##########################################################################################
#	__main__
# ##########################################################################################

usage = "usage: %prog [options] <tcxfile> <laplist>\n\n"
usage += "where:\n"
usage += "  <tcxfile>\tfile output from Garmin Training Center\n"
usage += "  <laplist> is one of two formats:\n"
usage += "     no switches\tsets of laps to be averaged, i.e., 5,3 means average first 5 laps, next 3 laps, then remaining\n"
usage += "     -interval\tnum laps before first workout then num repeats\n"

parser = optparse.OptionParser(usage=usage)
parser.add_option("-i", "--interval", dest="interval", action="store_true", help="treat input as intervals: num laps before first workout then num repeats")
parser.set_defaults(interval=False)
(options, args) = parser.parse_args()

tcxfile = args.pop(0)
laplist = args.pop(0)
sets = [int(si) for si in laplist.split(',')]
numsets = len(sets)

root = xmldict.readXmlFile(tcxfile)
laps = root['TrainingCenterDatabase']['Activities']['Activity']['Lap']

numlaps = len(laps)

# set up uselap (ul) array
ul = [True for i in range(numlaps)]

# if interval (-interval) switch is set, reset to use a single set, and update uselap (ul) array
# to force accumulation of only the indicated repeats
if (options.interval):
	ul = [False for i in range(numlaps)]
	(numbefore,numint) = sets
	for i in range(numint):
		ul[numbefore+i*2] = True

	sets = [numlaps,]
	numsets = 0


more = True
l = 0
tttime = 0
tthrtime = 0
tthr = 0
ttdist = 0

setstr = ""
csvstr = "distance,time,pace,avghr\n"
pacesstr = {}
for s in range(numsets+1):  # and more
	ttime = 0
	thrtime = 0
	thr = 0
	tdist = 0
	if s < numsets:
		setlimit = sets[s]
	else:
		setlimit = numlaps # numlaps for conventient ceiling for limit
	for i in range(setlimit): 	# and more; i++) {
		time = float(laps[l]['TotalTimeSeconds'])
		hr   = int(laps[l]['AverageHeartRateBpm']['Value'])
		dist = float(laps[l]['DistanceMeters'])		# bug in xmldict?  Seems to put into a list if any child tag has the same name
		
		if dist != 0.0:
		
			# accumulate for this set
			# for interval switch, only indicated laps are averaged
			if (ul[l]):
				ttime += time
				if (l!=0):
					thrtime += time 
					thr += hr * time
				
				tdist += dist
			
			
			# accumulate for whole run
			tttime += time
			if (l!=0):
				tthrtime += time
				tthr += hr * time
			
			ttdist += dist
			
			pacesstr[l] = pacesstr.setdefault(l,"") + timestr(time) + "({0:d})".format(hr)
			csvstr += "{0:.2f},{1},{2},{3}\n".format(dist/METERPMILE,timestr(time),pace(time,dist),hr)

			# indicate distance=pace for odd-distanced splits
			if (dist != METERPMILE):
				pacesstr[l] += " [{0:.2f}={1}]".format(dist/METERPMILE,pace(time,dist))

		# weird 0 distance lap
		else:
			pacesstr[l] = pacesstr.setdefault(l,"") + timestr(time) + "({0:d})".format(hr)
			
		l += 1
		if (l == numlaps):
			more = False
			break
	
	# calculate averages over this set
	if thrtime > 0:
		ahr = int ((thr / thrtime) + 0.5)
	else:
		ahr = "n/a"
	apace = pace (ttime, tdist)
	adist = tdist / METERPMILE
	if s < numsets:
		intmiles = int(adist+0.5)
		if math.fabs(tdist/intmiles-METERPMILE) < EPSILON:
			adistpr = int(adist+EPSILON/METERPMILE)
		else:
			adistpr = adist 
	else: 
		adistpr = "{0:.2f}".format(adist)
	
	if (s > 0):
		setstr += ", "
	setstr += "{0}@{1}({2})".format(adistpr,apace,ahr)

	if not more:
		break
	

# calculate averages over the whole run
athr = int ((tthr / tthrtime) + 0.5)
atpace = pace (tttime, ttdist)
atdist = ttdist / METERPMILE
totstr = "{0:.1f} miles, {1}, {2}/mi, AHR {3} ({4}% MHR)".format (atdist, timestr(tttime), atpace, athr, int((athr*100/MHR)+0.5))

CSV = open ("history.csv","w")
CSV.write (csvstr)
CSV.close()

OUT = open ("temp.txt","w")
OUT.write ("{0}\n".format(totstr))
OUT.write ("{0}\n\n".format(setstr))
if (options.interval):
	OUT.write ("splits:\n")
	for l in range(numlaps):
		if (ul[l]):
			OUT.write ("{0}\n".format(pacesstr[l]))
	OUT.write ("\n")

OUT.write ("all splits:\n")
for l in range(numlaps):
	split = l+1
	OUT.write ("{0} - {1}\n".format(split,pacesstr[l]))

OUT.close()
os.startfile ("temp.txt")



###########################################################################################
#   running - general purpose running related scripts
#
#   Date        Author      Reason
#   ----        ------      ------
#   01/08/13    Lou King    Create
#
#   Copyright 2012 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
running - general purpose running related scripts
===================================================
'''

# pypi

# github

# other

# home grown

# general purpose exceptions
class parameterError(Exception): pass
class accessError(Exception): pass




#!/usr/bin/python
# ##########################################################################################
#	comparegpx - compare two gpx files
#
#	Date		Author		Reason
#	----		------		------
#	08/29/12	Lou King	Create
#
# ##########################################################################################
'''
comparegpx - compare two gpx files
=======================================
'''

# standard
import pdb
import optparse
import datetime
import os.path
import csv
import collections

# pypi
from pykml.factory import KML_ElementMaker as KML
from pykml.factory import GX_ElementMaker as GX

# github
import gpxpy
import gpxpy.geo

# home grown
from loutilities import timeu 

METERPMILE = 1609.3439941
t = timeu.asctime('%Y-%m-%dT%H:%M:%SZ')

class invalidCoeff(Exception): pass

# ###############################################################################
def main():
# ###############################################################################

    usage = "usage: %prog [options] <gpxfile1> <gpxfile2>\n\n"
    usage += "where:\n"
    usage += "  <gpxfile>\tgpx formatted file"

    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-o", "--output", dest="output", help="output file", default=None)
    (options, args) = parser.parse_args()

    gpxfile = {}
    _GPX = {}
    gpx = {}
    for f in range(2):
        gpxfile[f] = args.pop(0)
        _GPX[f] = open(gpxfile[f],'r')
        gpx[f] = gpxpy.parse(_GPX[f])
        _GPX[f].close()

    # loop through gpx tracks, collecting data
    times = {}
    dists = {}
    for f in range(2):
        times[f] = []
        dists[f] = []
        lastpoint = None
        totdist = 0
        for track in gpx[f].tracks:
            for segment in track.segments:
                for point in segment.points:
                    if not lastpoint:
                        lastpoint = point
                    plon = point.longitude
                    plat = point.latitude
                    pelev = point.elevation
                    thisdist = gpxpy.geo.distance(lastpoint.latitude, lastpoint.longitude, lastpoint.elevation, plat, plon, pelev)
                    lastpoint = point
                    totdist += thisdist
                    dists[f].append(totdist)
                    times[f].append(timeu.dt2epoch(point.time))
    
    # analyze every 5 seconds of data, interpolating between points, for all overlapping tracks
    earliest = max(times[0][0], times[1][0])    #max forces intersection
    latest = min(times[0][-1], times[1][-1])    #min forces intersection
    earliest = (int(earliest+5)//5)*5    # round to next 5 second boundary
    latest = (int(latest)//5)*5          # round to last 5 second boundary
    
    results = {}
    for f in range(2):
        titer = iter(times[f])
        diter = iter(dists[f])
        tdeq = collections.deque([],2)
        ddeq = collections.deque([],2)
        try:
            for time in range(earliest,latest,5):
                if time not in results:
                    results[time] = {0:None, 1:None}

                while len(tdeq) < 2 or tdeq[-1] < time: # try to scan times to get mult 5 time between measured time points
                    tdeq.append(next(titer))
                    ddeq.append(next(diter))
                
                #pdb.set_trace()
                interpcoeff = float(time-tdeq[0]) / (tdeq[1]-tdeq[0])
                if interpcoeff < 0 or interpcoeff > 1:  # logic error if this is true
                    raise invalidCoeff
                    
                thisdist = ddeq[0] + interpcoeff*(ddeq[1]-ddeq[0])
                results[time][f] = thisdist
                
        except StopIteration:
            pass
    
    if options.output == None:
        outfile = os.path.basename(gpxfile[0]) + '.csv'
    else:
        outfile = options.output
    OUT = open(outfile,'w')
    OUT.write('time,{0},{1}\n'.format(gpxfile[0],gpxfile[1]))
    for time in range(earliest,latest,5):
        OUT.write('{0},{1},{2}\n'.format(t.epoch2asc(time),results[time][0],results[time][1]))
    OUT.close()

# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




#!/usr/bin/python
###########################################################################################
#   competitor - access methods for competitor.com
#
#   Date        Author      Reason
#   ----        ------      ------
#   11/23/13    Lou King    Create
#
#   Copyright 2013,2014 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
competitor - access methods for competitor.com
===================================================
'''

# standard
import argparse
import os.path
import urllib.request, urllib.parse, urllib.error
from bs4 import BeautifulSoup
import unicodedata
import logging
import pdb
import copy
logging.basicConfig(format='%(asctime)s %(levelname)s:%(message)s')

# pypi
import httplib2
from IPython.core.debugger import Tracer; debug_here = Tracer()

# github

# other

# home grown
from loutilities import timeu
from loutilities import csvu
from runningclub import render
from running import accessError, parameterError

# access stuff
PAGESIZE = 100
COMPETITOR_URL = 'http://running.competitor.com'
RESULTS_METHOD = 'cgiresults'
RESULTS_SEARCH = 'firstname={firstname}&lastname={lastname}&bib={bib}&gender={gender}&division={division}&city={city}&state={state}'
# RACE_RESULTS = 'eId={raceid}&eiId={yearid}&seId={eventid}'
#PAGING = 'resultsPage={pagenum}&rowCount={pagesize}'.format(pagesize=PAGESIZE)

HTTPTIMEOUT = 10
MPERMILE = 1609.344

tindate  = timeu.asctime('%m/%d/%Y %I:%M:%S %p')
toutdate = timeu.asctime('%Y-%m-%d')

#----------------------------------------------------------------------
def racenameanddist(soup):
#----------------------------------------------------------------------
    '''
    get race name and distance from soup
    
    :param soup: BeautifulSoup object for whole page
    :rtype: racename, distmiles, distkm
    '''

    ensoup = soup.find(class_='event-name')
    if ensoup:
        eventstring = ensoup.text
        rawparts = eventstring.split('-')
        eventparts = [p.strip() for p in rawparts]
        distfield = eventparts[-1].strip()
        # note might have been hyphen in event name
        racename = '-'.join(rawparts[0:-1]).strip()
    else:
        raise EventNotFound
        
    # accumulate distance, and figure out where units field starts
    dist = 0
    startunits = 0
    for digit in distfield:
        if not digit.isdigit():
            break
        dist *= 10
        dist += int(digit)
        startunits += 1
    
    # pull out the units from the distance field, and interpret
    units = distfield[startunits:].strip()
    
    # special cases
    if distfield == 'Marathon':
        distmiles = 26.21875    # true marathon
        distkm = distmiles * (MPERMILE/1000)
        
    elif distfield == 'Half Marathon' or distfield == '1/2 Marathon':
        distmiles = 13.109375   # true half marathon
        distkm = distmiles * (MPERMILE/1000)
        
    # kilometers
    elif units == 'K':
        distkm = dist 
        distmiles = (dist * 1000) / MPERMILE
    
    # miles
    elif units == 'Miler':
        # some special cases
        if dist == 13:
            distmiles = 13.109375   # true half marathon
        elif dist == 26:
            distmiles = 26.21875    # true marathon
        else:
            distmiles = dist
        distkm = distmiles * (MPERMILE/1000)
    
    else:
        distmiles = None
        distkm = None

    return racename,distmiles,distkm

#----------------------------------------------------------------------
def racedate(soup):
#----------------------------------------------------------------------
    '''
    get race date from soup
    
    :param soup: BeautifulSoup object for whole page
    :rtype: date of race
    '''

    edsoup = soup.find(class_='event-date')
    if edsoup:
        eventdate = edsoup.text.strip()
    else:
        eventdate = None

    # TODO: for now just return string - should return datetime object
    return eventdate

########################################################################
class competitorParse():
########################################################################
    '''
    gparseobject
    
    :param headingsoup: BeautifulSoup object for column header row
    :param get: _get function from competitor() to retrieve additional information
    :param racequery: dict with parameters to get correct race information
    :rtype: competitorParse instance
    '''

    #----------------------------------------------------------------------
    def __init__(self, headingsoup, get, racequery):
    #----------------------------------------------------------------------
        self.headings = []
        for col in headingsoup.find_all('td',recursive=False):
            self.headings.append(col.text)
        
        self.racequery = copy.copy(racequery)
        self.get = get

    #----------------------------------------------------------------------
    def details(self,detailurl):
    #----------------------------------------------------------------------
        '''
        get detail from url
        
        :param detailurl: url to retrieve result detail from
        :rtype: details dict
        '''
        
        # note detailurl may be relative, and if so needs to be added to base url
        if detailurl[0:7] != 'http://':
            detailurl = COMPETITOR_URL + detailurl

        detail = self.get(detailurl)
        detailsoup = BeautifulSoup(detail)
        
        # get age and gender
        agegen = detailsoup.find(class_='detail-pptage').text
        detailsdict = {}
        for attrval in agegen.split('|'):
            attr,val = [av.strip() for av in attrval.split(':')]
            detailsdict[attr] = val
        
        # get performance stats
        perfsoup = detailsoup.find(class_='detail-performance-stats')
        for li in perfsoup.find_all('li'):
            attr,val = [av.strip() for av in li.text.split(':')]
            # these vals are like '100 out of 15245'
            detailsdict['pf'+attr+'Place'],detailsdict['pf'+attr+'Count'] = [vo.strip() for vo in val.split('out of')]
        
        # make values integer if possible
        for attr in detailsdict:
            try:
                detailsdict[attr] = int(detailsdict[attr])
            except ValueError:
                pass

        return detailsdict
    
    #----------------------------------------------------------------------
    def result(self,rowsoup):
    #----------------------------------------------------------------------
        '''
        get result from rowsoup
        
        :param headingsoup: BeautifulSoup object for column header row
        :param rowsoup: BeautifulSoup object for row
        :rtype: competitorResult instance
        '''
    
        # pull rowcell text out of result row
        # save the soup cells so we can pull in the participant details link
        rowcells = []
        soupcells = []
        for cell in rowsoup.find_all('td'):
            rowcells.append(cell.text.strip())
            soupcells.append(cell)
            
        soupdict = dict(list(zip(self.headings,soupcells)))
        rowdict = dict(list(zip(self.headings,rowcells)))
        
        # try to translate integers
        for attr in rowdict:
            try:
                rowdict[attr] = int(rowdict[attr])
            except ValueError:
                pass
            
        # pull in details
        detailurl = soupdict['Name'].find('a')['href']
        details = self.details(detailurl)
        rowdict.update(details)
        
        # convert to competitorResult object
        irsattrs = competitorResult.result_attrs
        orsattrs = competitorResult.attrs
        attrvals = []
        for i in range(len(irsattrs)):
            inattr = irsattrs[i]
            outattr = orsattrs[i]
            attrvals.append((outattr,rowdict[inattr]))
        
        rowresult = competitorResult()
        rowresult.set(attrvals)
        
        return rowresult
    
########################################################################
class competitorResult():
########################################################################
    '''
    holds result from competitor.com
    
    :param oaplace: overall place
    :param genplace: gender place
    :param age: age on race day
    :param gender: gender
    :param racetime: finishing time h:mm:ss
    :param racedate: date of race yyyy-mm-dd
    :param raceloc: location of race
    :param racename: name of race
    :param distmiles: distance in miles
    :param distkm: distance in kilometers
    '''
    # result_attrs are within the response from competitor.com
    # attrs must be in the same order
    # loop needs to be driven by result_attrs because it's shorter than output attrs
    # racename and distance fields are determined from parsing of 'eventname' from competitor.com
    result_attrs = 'pfOverallPlace,pfGenderPlace,pfDivisionPlace,Name'.split(',') + ['City, State'] + 'Age,Gender,Time'.split(',')    
    attrs = 'oaplace,genplace,divplace,name,hometown,age,gender,racetime,racedate,raceloc,racename,distmiles,distkm'.split(',')
    
    #----------------------------------------------------------------------
    def __init__(self,oaplace=None,genplace=None,divplace=None,name=None,hometown=None,age=None,gender=None,racetime=None,racedate=None,raceloc=None,racename=None,distmiles=None,distkm=None):
    #----------------------------------------------------------------------
        self.oaplace = oaplace
        self.genplace = genplace
        self.divplace = divplace
        self.name = name
        self.hometown = hometown
        self.age = age
        self.gender = gender
        self.racetime = racetime
        self.racedate = racedate
        self.raceloc = raceloc
        self.racename = racename
        self.distmiles = distmiles
        self.distkm = distkm

    #----------------------------------------------------------------------
    def __repr__(self):
    #----------------------------------------------------------------------
        reprval = '{}('.format(self.__class__)
        for attr in self.attrs:
            reprval += '{}={},'.format(attr,getattr(self,attr))
        reprval = reprval[:-1]
        reprval += ')'
        return reprval
    
    #----------------------------------------------------------------------
    def set(self,attrvals):
    #----------------------------------------------------------------------
        '''
        set attributes based on list of attr,val pairs
        
        :param attrvals: [(attr,val),...]
        '''
        
        for attr,inval in attrvals:
            val = csvu.str2num(inval)
            
            # special processing for dates
            if attr in ['racedate']:
                val = toutdate.epoch2asc(tindate.asc2epoch(val))
                
            setattr(self,attr,val)

########################################################################
class Competitor():
########################################################################
    '''
    access methods for competitor.com
    '''

    #----------------------------------------------------------------------
    def __init__(self,debug=False):
    #----------------------------------------------------------------------
        """
        initialize http 
        """
        # need http object
        self.http = httplib2.Http(timeout=HTTPTIMEOUT)

        # set up logging level
        self.log = logging.getLogger('running.competitor')
        self.setdebug(debug)
        
        # count how many pages have been retrieved
        self.urlcount = 0
        
        # query race
        self.racequery = {'eId':'','eiId':'','seId':''}
        
    #----------------------------------------------------------------------
    def setdebug(self,debugval):
    #----------------------------------------------------------------------
        '''
        set debugging attribute for this class
        
        :param debugval: set to True to enable debugging
        '''
        if not debugval:
            level = logging.INFO
        else:
            level = logging.DEBUG
        self.log.setLevel(level)
        
    #----------------------------------------------------------------------
    def setraceyear(self,eventid,eventinstanceid,singleeventid):
    #----------------------------------------------------------------------
        '''
        set the raceid, yearid and eventid for subsequent requests
        
        :param eventid: event id from running.competitor.com
        :param eventinstanceid: event instance id from running.competitor.com
        :param singleeventid: single event id from running.competitor.com
        '''
        self.racequery['eId'] = eventid
        self.racequery['eiId'] = eventinstanceid
        self.racequery['seId'] = singleeventid

    #----------------------------------------------------------------------
    def geturlcount(self):
    #----------------------------------------------------------------------
        '''
        each time a url is retrieved, this counter is bumped
        
        :rtype: integer, number of url's retrieved
        '''
        return self.urlcount

    #----------------------------------------------------------------------
    def getresults(self,limit=None):
    #----------------------------------------------------------------------
        '''
        return results for the current race / event
        
        :param limit: limit number of records (for testing only)
        :rtype: list of competitorResult objects
        '''
        
        # get the first page
        first = True
        pagenum = 1
        
        results = []
        while True:
            params = {'resultsPage':pagenum, 'rowCount':PAGESIZE}
            params.update(self.racequery)
            self.log.info(params)
            pagenum += 1
            
            page = self._get(RESULTS_METHOD,**params)
            
            # parse the page
            soup = BeautifulSoup(page)
            
            # first time through, grab the event, distance and date
            if first:
                event,distmiles,distkm = racenameanddist(soup)
                eventdate = racedate(soup)
                first = False
                
            # pull out the rows of results
            # .select returns a list, so take the first entry
            resulttable = soup.select('.cg-runnergrid-table tbody')[0]
            if not resulttable:
                raise ResultsNotFound
            
            # get all the rows in the table
            resultrows = resulttable.find_all('tr',recursive=False)
    
            # get out when there's no data in this page
            if len(resultrows) <= 1: break
            
            # interpret the row information into the rows list
            headerrow = resultrows[0]
            
            # initialize row parse object with header information, and access to competitor.com (self)
            # TODO: probably the _get, _geturl should be encapsulated in a class on it's own, as multiple classes use it
            rr = competitorParse(headerrow,self._geturl,self.racequery)
            
            # save each result in results
            for resultrow in resultrows[1:]:
                result = rr.result(resultrow)
                result.racename = event
                result.distmiles = distmiles
                result.distkm = distkm
                results.append(result)
                
                # break out if testing limit reached
                if limit:
                    if len(results) >= limit: break
            
            # break out if testing limit reached
            if limit:
                if len(results) >= limit: break
        
        # and back to caller
        return results
        
    #----------------------------------------------------------------------
    def _geturl(self,url):
    #----------------------------------------------------------------------
        '''
        get raw url
        
        :param url: url to retrieve
        :rtype: content (html)
        '''
        # loop RETRIES times for timeout
        retries = 10
        while retries > 0:
            retries -= 1
            try:
                self.log.debug(url)
                resp,content = self.http.request(url)
                self.urlcount += 1
                break
            except Exception as e:
                if retries == 0:
                    self.log.info('{} requests attempted'.format(self.geturlcount()))
                    self.log.error('http request failure, retries exceeded: {0}'.format(e))
                    raise
                self.log.warning('http request failure: {0}'.format(e))
        
        if resp.status != 200:
            raise accessError('URL response status = {0}'.format(resp.status))
        
        return content

    #----------------------------------------------------------------------
    def _get(self,method='',**params):
    #----------------------------------------------------------------------
        '''
        get method for competitor access
        
        :param method: competitor method to call
        :param **params: parameters for the method
        :rtype: content (html)
        '''
        
        body = urllib.parse.urlencode(params)
        url = '{}/{}?{}'.format(COMPETITOR_URL,method,body)
        content = self._geturl(url)
        
        return content 
        
#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    unit test for competitor.py
    '''
    
    parser = argparse.ArgumentParser(description=descr,formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running',version.__version__))
    args = parser.parse_args()

    # this would be a good place for unit tests
    
# ##########################################################################################
#	__main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
###########################################################################################
# runningaheadmembers -- class pulls in individual memberships for later processing
#
#       Date            Author          Reason
#       ----            ------          ------
#       04/11/15        Lou King        Create
#
#   Copyright 2015 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################

# standard
import csv
from datetime import datetime
import difflib

# home grown
from loutilities import timeu
ymd = timeu.asctime('%Y-%m-%d')
from loutilities.csvwt import wlist

class unsupportedFileType(): pass

########################################################################
class RunningAheadMember():
########################################################################
    '''
    Represents single RunningAHEAD member

    :param membership: membership record from RunningAHEAD export file
    '''

    # translate selected fields of file header to attributes we want to keep
    filehdr = 'MembershipType,FamilyName,GivenName,MiddleName,Gender,DOB,Email,EmailOptIn,PrimaryMember,RenewalDate,JoinDate,ExpirationDate,Street1,Street2,City,State,PostalCode,Country,Telephone,EntryType'.split(',')
    memberattr = 'membershiptype,lname,fname,mname,gender,dob,email,emailoptin,primarymember,renew,join,expiration,street1,street2,city,state,zip,country,telephone,entrytype'.split(',')
    memberxlate = dict(list(zip(filehdr,memberattr)))

    # only repr these fields
    reprattr = 'fname,lname,dob,join,renew,expiration'.split(',')

    #----------------------------------------------------------------------
    def __init__(self,membership):
    #----------------------------------------------------------------------

        for key in self.filehdr:
            if key in self.memberxlate:
                setattr(self,self.memberxlate[key],membership[key])

    #----------------------------------------------------------------------
    def __repr__(self):
    #----------------------------------------------------------------------

        reprval = '{}('.format(self.__class__)
        for attr in self.reprattr:
            if attr[0:2] == '__' or attr == 'fields': continue
            reprval += '{}={},'.format(attr,getattr(self,attr))
        reprval = reprval[:-1]  #remove trailing comma
        reprval += ')'
        return reprval



########################################################################
class RunningAheadMembers():
########################################################################
    '''
    Collect member data from RunningAHEAD individual membership export
    file, containing records from the beginning of the club's member
    registration until present.

    Provide access functions to gain access to these membership records.

    :param memberfile: member filename, filehandle or string of file records
    :param overlapfile: debug file to test for overlaps between records
    '''

    #----------------------------------------------------------------------
    def __init__(self,memberfile,overlapfile=None):
    #----------------------------------------------------------------------

        # initialize class attributes
        self.closematches = None
        self.names = {}
        self.dobnames = {}

        # check for type of memberfile, assume not opened here
        openedhere = False

        # if str, assume this is the filename
        if isinstance(memberfile, str):
            memberfileh = open(memberfile, 'r', newline='')
            openedhere = True

        # if file, remember handle
        elif isinstance(memberfile, file):
            memberfileh = memberfile

        # if list, it works like a handle
        elif type(memberfile) in [list,wlist]:
            memberfileh = memberfile

        # otherwise, not handled
        else:
            raise unsupportedFileType

        # input is csv file
        INCSV = csv.DictReader(memberfileh)
        
        ## preprocess file to remove overlaps between join date and expiration date across records
        # each member's records are appended to a list of records in dict keyed by (lname,fname,dob)
        for membership in INCSV:
            asc_joindate = membership['JoinDate']
            asc_expdate = membership['ExpirationDate']
            fname = membership['GivenName']
            lname = membership['FamilyName']
            dob = membership['DOB']
            memberid = membership['MemberID']
            fullname = '{}, {}'.format(lname,fname)

            # get list of records associated with each member, pulling out significant fields
            thisrec = {'MemberID':memberid,'name':fullname,'join':asc_joindate,'expiration':asc_expdate,'dob':dob,'fullrec':membership,'RunningAheadMember':RunningAheadMember(membership)}
            # careful - this tuple order is assumed in several places
            thisname = (lname,fname,dob)
            if not thisname in self.names:
                self.names[thisname] = []
            self.names[thisname].append(thisrec)

        #debug
        if overlapfile:
            _OVRLP = open(overlapfile,'w',newline='')
            OVRLP = csv.DictWriter(_OVRLP,['MemberID','name','dob','renewal','join','expiration','tossed'],extrasaction='ignore')
            OVRLP.writeheader()

        # sort list of records under each name, and remove overlaps between records
        # create dobnames access from self.names
        # self.dobnames allows access to self.names -- self.dobnames[dob] can be used to find key for self.names[lname,fname,dob]
        for thisname in self.names:
            # self.dobnames allows access to self.names key based on dob
            lname,fname,dob = thisname
            if dob not in self.dobnames:
                self.dobnames[dob] = []
            if (lname,fname) not in self.dobnames[dob]:
                self.dobnames[dob].append({'lname':lname,'fname':fname})

            # sort should result so records within a name are by join date within expiration year
            # see http://stackoverflow.com/questions/72899/how-do-i-sort-a-list-of-dictionaries-by-values-of-the-dictionary-in-python
            self.names[thisname] = sorted(self.names[thisname],key=lambda k: (k['expiration'],k['join']))
            toss = []
            for i in range(1,len(self.names[thisname])):
                # if overlapped record detected, push this record's join date after last record's expiration
                # note this only works for overlaps across two records -- if overlaps occur across three or more records that isn't detected
                # this seems ok as only two record problems have been seen so far
                if self.names[thisname][i]['join'] <= self.names[thisname][i-1]['expiration']:
                    lastexp_dt = ymd.asc2dt(self.names[thisname][i-1]['expiration'])
                    thisexp_dt = ymd.asc2dt(self.names[thisname][i]['expiration'])
                    jan1_dt = datetime(lastexp_dt.year+1,1,1)
                    jan1_asc = ymd.dt2asc(jan1_dt)
            
                    # ignore weird record anomalies where this record duration is fully within last record's
                    if jan1_dt > thisexp_dt:
                        toss.append(i)
                        self.names[thisname][i]['tossed'] = 'Y'
            
                    # debug
                    if overlapfile:
                        OVRLP.writerow(self.names[thisname][i-1])    # this could get written multiple times, I suppose
                        OVRLP.writerow(self.names[thisname][i])
            
                    # update this record's join dates
                    self.names[thisname][i]['join'] = jan1_asc
                    self.names[thisname][i]['fullrec']['JoinDate'] = jan1_asc
                    self.names[thisname][i]['RunningAheadMember'].join = jan1_asc
            
            # throw out anomalous records. reverse toss first so the pops don't change the indexes.
            toss.reverse()
            for i in toss:
                self.names[thisname].pop(i)

        # close the debug file if present
        if overlapfile:
            _OVRLP.close()

        # close the file if opened here
        if openedhere:
            memberfileh.close()

    #----------------------------------------------------------------------
    def membership_iter(self,raw = False):
    #----------------------------------------------------------------------
    # TODO: is raw needed?
        '''
        generator function that yields full record for each of memberships

        :param raw: True to yield dict, False to yield RunningAheadMember object, default False
        '''
        for thisname in self.names:
            for thismembership in self.names[thisname]:
                if not raw:
                    yield thismembership['RunningAheadMember']
                else:
                    yield thismembership['fullrec']

    #----------------------------------------------------------------------
    def member_iter(self,raw=False):
    #----------------------------------------------------------------------
    # TODO: is raw needed?
        '''
        generator function that yields latest membership record for each of names with
        JoinDate updated to earliest JoinDate

        :param raw: True to yield dict, False to yield RunningAheadMember object, default False
        '''
        for thisname in self.names:
            if not raw:
                thismembership = self.names[thisname][-1]['RunningAheadMember']
                thismembership.join = self.names[thisname][0]['join']
                yield thismembership
            else:
                thismembership = self.names[thisname][-1]['fullrec']
                thismembership['JoinDate'] = self.names[thisname][0]['join']
                yield thismembership

    #----------------------------------------------------------------------
    def getmember(self, memberkey):
    #----------------------------------------------------------------------
        '''
        retrieve latest membership record for memberkey with 
        JoinDate updated to earliest JoinDate

        :param memberkey: (lname,fname,dob)
        :rtype: RunningAheadMember object
        '''

        thismembership = self.names[memberkey][-1]['RunningAheadMember']
        thismembership.join = self.names[memberkey][0]['join']
        return thismembership

    #----------------------------------------------------------------------
    def getmemberships(self, memberkey):
    #----------------------------------------------------------------------
        '''
        retrieve list of membership records for memberkey

        :param memberkey: (lname,fname,dob)
        :rtype: [RunningAheadMember object, ...]
        '''

        thesememberships = []
        for thismembership in self.names[memberkey]:
            thesememberships.append(thismembership['RunningAheadMember'])
        return thesememberships

    #----------------------------------------------------------------------
    def getmemberkey(self, lname, fname, dob, cutoff=0.6, n=10):
    #----------------------------------------------------------------------
        '''
        retrieve member key based on name, dob

        if name wasn't found, None is returned
        if None is returned, check close matches using getclosematches()
        
        :param lname: last name
        :param fname: first name
        :param dob: date of birth yyyy-mm-dd
        :param cutoff: float in range (0,1] ratio of closeness to match name
        :param n: number of names checked for closeness (most similar)
        :rtype: (lname,fname,dob) or None if not found
        '''
        # no matches missed yet
        self.closematches = []

        # check dob
        if dob not in self.dobnames:
            return None

        # make list of members with this dob
        nameskeys = [('{} {}'.format(n['fname'],n['lname']).lower(),n) for n in self.dobnames[dob]]
        possiblenames = [nk[0] for nk in nameskeys]
        possiblekeys  = [(nk[1]['lname'],nk[1]['fname'],dob) for nk in nameskeys]

        # make list of close matches
        searchname = '{} {}'.format(fname,lname).lower()
        closematches = difflib.get_close_matches(searchname,possiblenames,n=10,cutoff=cutoff)
        for match in closematches:
            # this should be first time through loop, so self.closematches will be empty list
            if match == searchname:
                return possiblekeys[possiblenames.index(match)]
            else:
                self.closematches.append(possiblekeys[possiblenames.index(match)])

        # didn't find exact match
        return None


    #----------------------------------------------------------------------
    def getclosematchkeys(self):
    #----------------------------------------------------------------------
        '''
        can be called after findmembers() to return list of members found, but did not match exactly
        
        :rtype: [(lname,fname,dob), ...]
        '''
        
        return self.closematches



#!/usr/bin/python
###########################################################################################
# runningaheadparticipants -- class pulls in individual registrations for later processing
#
#       Date            Author          Reason
#       ----            ------          ------
#       04/11/15        Lou King        Create
#
#   Copyright 2015 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################

# standard
import csv

# home grown

########################################################################
class RunningAheadParticipant():
########################################################################
    '''
    Represents single RunningAHEAD registration

    :param registration: registration record from RunningAHEAD export file
    '''

    # translate selected fields of file header to attributes we want to keep
    'Participant ID,Registration Date,Event Category,Status,Bib,Last Name,First Name,Middle Name,Gender,Age,DOB,Email,Street 1,Street 2,City,State,ZIP Code,Country'

    filehdr = 'Registration Date,Event Category,Status,Bib,Last Name,First Name,Middle Name,Gender,Age,DOB,Email,Street 1,Street 2,City,State,ZIP Code,Country'.split(',')
    participantattr = 'registrationdate,category,status,bib,lname,fname,mname,gender,age,dob,email,street1,street2,city,state,zip,country'.split(',')
    participantxlate = dict(list(zip(filehdr,participantattr)))

    # only repr these fields
    reprattr = 'fname,lname,dob,registrationdate'.split(',')

    #----------------------------------------------------------------------
    def __init__(self,registration):
    #----------------------------------------------------------------------

        for key in self.filehdr:
            if key in self.participantxlate:
                setattr(self,self.participantxlate[key],registration[key])

    #----------------------------------------------------------------------
    def __repr__(self):
    #----------------------------------------------------------------------

        reprval = '{}('.format(self.__class__)
        for attr in self.reprattr:
            if attr[0:2] == '__' or attr == 'fields': continue
            reprval += '{}={},'.format(attr,getattr(self,attr))
        reprval = reprval[:-1]  #remove trailing comma
        reprval += ')'
        return reprval



########################################################################
class RunningAheadParticipants():
########################################################################
    '''
    Collect participant data from RunningAHEAD event registration export
    file.

    Provide access functions to gain access to these registration records.

    :param participantfile: participant filename, filehandle or string of file records
    :param overlapfile: debug file to test for overlaps between records
    '''

    #----------------------------------------------------------------------
    def __init__(self,participantfile,overlapfile=None):
    #----------------------------------------------------------------------

        # check for type of participantfile, assume not opened here
        openedhere = False

        # if str, assume this is the filename
        if isinstance(participantfile, str):
            participantfileh = open(participantfile, 'r', newline='')
            openedhere = True

        # if file, reparticipant handle
        elif isinstance(participantfile, file):
            participantfileh = participantfile

        # if list, it works like a handle
        elif isinstance(participantfile, list):
            participantfileh = participantfile

        # otherwise, not handled
        else:
            raise unsupportedFileType

        # input is csv file
        INCSV = csv.DictReader(participantfileh)
        
        # pull in each record in the file
        self.registrations = {}
        for registration in INCSV:
            thisparticipant = RunningAheadParticipant(registration)
            lname = thisparticipant.lname
            fname = thisparticipant.fname
            dob = thisparticipant.dob

            # get list of records associated with each participant, pulling out significant fields
            thisrec = {'lname':lname,'fname':fname,'dob':dob,'fullrec':registration,'RunningAheadParticipant':thisparticipant}
            thisname = (thisparticipant.lname,thisparticipant.fname,thisparticipant.dob)
            self.registrations[thisname] = thisrec

    #----------------------------------------------------------------------
    def allregistrations_iter(self):
    #----------------------------------------------------------------------
        '''
        generator function that yields full record for each registrations
        '''
        for thisregistration in self.registrations:
            yield self.registrations[thisregistration]['RunningAheadParticipant']

    #----------------------------------------------------------------------
    def activeregistrations_iter(self):
    #----------------------------------------------------------------------
        '''
        generator function that yields full record for each registrations
        '''
        for thisregistration in self.registrations:
            if self.registrations[thisregistration]['RunningAheadParticipant'].status == 'Registered':
                yield self.registrations[thisregistration]['RunningAheadParticipant']




#!/usr/bin/python
# ##########################################################################################
#   racewx - determine weather for race day
#
#	Date		Author		Reason
#	----		------		------
#	08/29/12	Lou King	Create
#
# ##########################################################################################
'''
racewx - determine weather for race day
===================================================
'''

# standard
import argparse
import urllib.request, urllib.parse, urllib.error
import json
import csv
import time
import math
import os.path

# pypi
import pytz # also consider https://pypi.python.org/pypi/gaepytz (tuned for google app engine)
import httplib2

# github
import gpxpy
import gpxpy.geo

# home grown
from running.running import version
from running import *
from loutilities import timeu
from loutilities import apikey

_ak = apikey.ApiKey('Lou King','running')
try:
    _FORECASTIOKEY = _ak.getkey('forecastio')
except apikey.unknownKey:
    print("'forecastio' key needs to be configured using apikey")
    raise

WXPERIOD = 5*60    # number of seconds between weather assessments
#tgpx = timeu.asctime('%Y-%m-%dT%H:%M:%S%Z')

HTTPTIMEOUT = 5
HTTPTZ = httplib2.Http(timeout=HTTPTIMEOUT)
HTTPWX = httplib2.Http(timeout=HTTPTIMEOUT,disable_ssl_certificate_validation=True)

# dewpoint from http://www.meteo-blog.net/2012-05/dewpoint-calculation-script-in-python/
# dewpoint constants
DPa = 17.271
DPb = 237.7 # degC

# heat index from http://en.wikipedia.org/wiki/Heat_index
# heat index constants
HIc = [None,-42.379,2.04901523,10.14333127,-0.22475541,-6.83783e-3,-5.481717e-2,1.22874e-3,8.5282e-4,-1.99e-6]

#----------------------------------------------------------------------
def dewpoint(temp,humidity):
#----------------------------------------------------------------------
    '''
    approximate dewpoint from temp and relative humidity
    from http://www.meteo-blog.net/2012-05/dewpoint-calculation-script-in-python/
    
    :param temp: temperature (fahrenheit)
    :param humidity: relative humidity (1-100)
    :rtype: dewpoint (fahrenheit)
    '''
    
    tempC = celsius(temp)
    Td = (DPb * gamma(tempC,humidity)) / (DPa - gamma(tempC,humidity))
 
    return fahrenheit(Td)
 
#---------------------------------------------------------------------- 
def gamma(tempC,humidity):
#----------------------------------------------------------------------
    '''
    gamma function for dewpoint calculation
    from http://www.meteo-blog.net/2012-05/dewpoint-calculation-script-in-python/
    
    :param tempC: temperature (celcius)
    :param humidity: relative humidity (1-100)
    '''
 
    g = (DPa * tempC / (DPb + tempC)) + math.log(humidity/100.0)
 
    return g
 
#----------------------------------------------------------------------
def windchill(temp,windspeed):
#----------------------------------------------------------------------
    '''
    wind chill calculation
    from http://en.wikipedia.org/wiki/Wind_chill#North_American_and_UK_wind_chill_index
    
    :param temp: temperature (fahrenheit)
    :param windspeed: wind speed (mph)
    :rtype: windchill (fahrenheit) or None if temp > 50 or windspeed < 3 mph
    '''
    
    if temp <= 50 and windspeed >= 3.0:
        return 35.74 + 0.6215*temp - 35.75*windspeed**0.16 + 0.4275*temp*windspeed**0.16
    else:
        return None

#----------------------------------------------------------------------
def heatindex(temp,humidity):
#----------------------------------------------------------------------
    '''
    heat index calculation
    from http://en.wikipedia.org/wiki/Heat_index#Formula
    
    :param temp: temperature (fahrenheit)
    :param humidity: relative humidity (0-100)
    :rtype: heat index (fahrenheit) or None if temp < 80 or humidity < 40
    '''
    
    if temp >= 80 and humidity >= 40:
        return (HIc[1] + HIc[2]*temp + HIc[3]*humidity + HIc[4]*temp*humidity + HIc[5]*temp**2 
                + HIc[6]*humidity**2 + HIc[7]*temp**2*humidity + HIc[8]*temp*humidity**2 + HIc[9]*temp**2*humidity**2)
    else:
        return None

#----------------------------------------------------------------------
def celsius(temp):
#----------------------------------------------------------------------
    '''
    convert Fahrenheit temp to Celcius
    
    :param temp: temperature in Fahrenheit
    :rtype: temperature in Celcius
    '''
    return (temp - 32) / (9.0/5.0)

#----------------------------------------------------------------------
def fahrenheit(temp):
#----------------------------------------------------------------------
    '''
    convert Celcius temp to Fahrenheit
    
    :param temp: temperature in Celcius
    :rtype: temperature in Fahrenheit
    '''
    return (temp * (9.0/5.0)) + 32

#----------------------------------------------------------------------
def gettzid(lat,lon):
#----------------------------------------------------------------------
    '''
    get time zone name based on lat, lon
    uses google maps api
    
    :param lat: latitude in decimal degrees
    :param lon: longitude in decimal degrees
    '''

    params = {'location':'{lat},{lon}'.format(lat=lat,lon=lon),
              'timestamp':0,
              'sensor':'true'}
    body = urllib.parse.urlencode(params)
    url = 'https://maps.googleapis.com/maps/api/timezone/json?{body}'.format(body=body)
    resp,jsoncontent = HTTPTZ.request(url)

    if resp.status != 200:
        raise accessError('URL response status = {0}'.format(resp.status))
    
    # unmarshall the response content
    content = json.loads(jsoncontent)

    if content['status'] != 'OK':
        raise accessError('URL content status = {0}'.format(content['status']))
    
    return content['timeZoneId']
    
#----------------------------------------------------------------------
def getwx(lat,lon,etime):
#----------------------------------------------------------------------
    '''
    get weather from forecast.io for specified latitude, longitude, time
    
    :param lat: latitude
    :param long: longitude
    :param etime: time in unix format (that is, seconds since midnight GMT on 1 Jan 1970)
    :rtype: weather dict for that location, time, e.g., {u'temperature': 46.59, u'precipType': u'rain', u'humidity': 0.62, u'cloudCover': 0.53, u'summary': u'Mostly Cloudy', u'pressure': 1014.87, u'windSpeed': 8.63, u'visibility': 10, u'time': 1366036200, u'windBearing': 326, u'icon': u'partly-cloudy-day'}
    '''
    
    url = 'http://api.forecast.io/forecast/{apikey}/{lat},{lon},{time}'.format(apikey=_FORECASTIOKEY,lat=lat,lon=lon,time=etime)
    resp,jsoncontent = HTTPWX.request(url)
    
    if resp.status != 200:
        raise accessError('URL response status = {0}'.format(resp.status))
    
    # unmarshall the response content, and return the weather for that time
    content = json.loads(jsoncontent)
    return content['currently']

#----------------------------------------------------------------------
def main():
#----------------------------------------------------------------------

    parser = argparse.ArgumentParser(version='{0} {1}'.format('running', version.__version__))
    parser.add_argument('gpxfile',help='gpx formatted file')
    parser.add_argument('racestarttime',help="time of race start in '%%Y-%%m-%%dT%%H:%%M' format")
    parser.add_argument('-o','--output',help='name of output file (default %(default)s)',default='racewx.csv')
    args = parser.parse_args()

    gpxfile = args.gpxfile
    racestarttime = args.racestarttime
    timrace = timeu.asctime('%Y-%m-%dT%H:%M')
    racestartdt = timrace.asc2dt(racestarttime) # naive
    output = args.output
    
    # get input
    _GPX = open(gpxfile,'r')
    gpx = gpxpy.parse(_GPX)

    # loop through gpx tracks
    wxdata = []
    lasttime = None
    exectime = int(round(time.time()))
    for track in gpx.tracks:
        for segment in track.segments:
            for point in segment.points:
                pepoch = timeu.dt2epoch(point.time)
                if not lasttime or pepoch-lasttime >= WXPERIOD:
                    plon = point.longitude
                    plat = point.latitude
                    if not lasttime:
                        starttime = pepoch
                        tzid = gettzid(plat, plon)
                        tz = pytz.timezone(tzid)
                        racestartlocdt = tz.normalize(tz.localize(racestartdt))
                        racestartepoch = timeu.dt2epoch(racestartlocdt)
                        shift = racestartepoch - starttime
                    targtime = timeu.dt2epoch(point.time)+shift # shift to race time
                    
                    # get weather
                    # temp, dew point, cloud cover, precip intensity
                    # wind speed/bearing: http://matplotlib.org/api/pyplot_api.html#matplotlib.pyplot.barbs
                    #   (http://matplotlib.1069221.n5.nabble.com/plot-arrows-for-wind-direction-degrees-td13499.html)
                    wx = getwx(plat,plon,targtime)
                    wx['lat'] = plat
                    wx['lon'] = plon
                    wx['dewpoint'] = dewpoint(wx['temperature'],wx['humidity']*100)
                    wx['windchill'] = windchill(wx['temperature'],wx['windSpeed'])
                    wx['heatindex'] = heatindex(wx['temperature'],wx['humidity']*100)
                    wxdata.append(wx)
                    
                    lasttime = pepoch

    # create the file and write header if it doesn't exist
    if not os.path.exists(output):
        writeheader = True
        _WX = open(output,'w',newline='')
    else:
        writeheader = False        
        _WX = open(output,'a',newline='')
        
    heading = ['exectime', 'time', 'lat', 'lon', 'temperature', 'humidity', 'dewpoint', 'windchill', 'heatindex', 'precipType', 'precipProbability', 'precipIntensity', 'windSpeed', 'windBearing', 'cloudCover', 'summary', 'pressure', 'visibility']
    WX = csv.DictWriter(_WX,heading,extrasaction='ignore')
    if writeheader:
        WX.writeheader()
    for wx in wxdata:
        wx['exectime'] = exectime
        WX.writerow(wx)
    _WX.close()

# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




#!/usr/bin/python
###########################################################################################
#   strava - access methods for strava.com
#
#   Date        Author      Reason
#   ----        ------      ------
#   09/08/15    Lou King    Create
#
#   Copyright 2016 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
strava - access methods for strava.com
===================================================
'''

# standard
import argparse
import json
import time
import logging
from tempfile import NamedTemporaryFile
import sys
import os.path
from collections import OrderedDict, defaultdict

# pypi
import requests

# github

# other

# home grown
from running.running import version
from loutilities import apikey
from loutilities import timeu
from loutilities.csvwt import record2csv
from loutilities.csvu import unicode2ascii

stravatime = timeu.asctime('%Y-%m-%dT%H:%M:%SZ')

KMPERMILE = 1.609344

DATEFIELD = 'start_date'

# from https://strava.github.io/api/v3/activities/
xworkout_type = {
    None : 'default',
    0    : 'default',
    1    : 'race',
    2    : 'long run',
    3    : 'workout',
    10   : 'default',
    11   : 'race',
    12   : 'workout',
}
workout_type = defaultdict(lambda: 'unknown',**xworkout_type)

#----------------------------------------------------------------------
def dist2miles(distance):
#----------------------------------------------------------------------
    '''
    convert distance to miles for distance returned from strava (meters)
    
    :param distance: distance field from strava
    :rtype: float - number of miles represented by the distance field
    '''
    mpermile = KMPERMILE * 1000
    
    distmiles = distance / mpermile
    
    return distmiles

########################################################################
class Strava():
########################################################################
    '''
    access methods for Strava.com

    :param cachefilename: name of cache file
    '''

    #----------------------------------------------------------------------
    def __init__(self, clubactivitycachefilename=None, debug=False, key=None):
    #----------------------------------------------------------------------
        """
        initialize 
        """

        # get credentials from configuration
        if not key:
            ak = apikey.ApiKey('Lou King','running')
            try:
                # key = ak.getkey('stravakey')
                # secret = ak.getkey('stravasecret')
                user = ak.getkey('stravauser')
            except apikey.unknownKey:
                raise parameterError("'stravauser' needs to be configured using apikey")
        else:
            user = key
        
        self.user = user

        # set up debug logging, if desired
        if debug:
            logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
            logging.getLogger().setLevel(logging.DEBUG)
            requests_log = logging.getLogger("requests.packages.urllib3")
            requests_log.setLevel(logging.DEBUG)
            requests_log.propagate = True

        # bring in clubactivitycache file, if requested
        self.clubactivitycache = {}
        self.clubactivitycachefilename = clubactivitycachefilename
        if self.clubactivitycachefilename:
            # only read clubactivitycache if file exists
            if os.path.isfile(clubactivitycachefilename):
                with open(clubactivitycachefilename,'r') as clubactivitycachefile:
                    # members are stored one per line, in json format
                    for line in clubactivitycachefile:
                        activity = json.loads(line)
                        id = activity['id']
                        # make sure there are no duplicates initially
                        if id not in self.clubactivitycache:
                            self.clubactivitycache[id] = activity
        
        # keep track of size of cache and number of activities added
        self.clubactivitycachesize = len(self.clubactivitycache)
        self.clubactivitycacheadded = 0
        
    #----------------------------------------------------------------------
    def close(self):
    #----------------------------------------------------------------------
        '''
        close the connection when we're done, and save the cache
        '''
        # save the cache in a temporary file, if requested and it's been updated
        if self.clubactivitycachefilename and self.clubactivitycacheadded > 0:
            # get full path for self.clubactivitycachefilename to assure cachedir isn't relative
            cachedir = os.path.dirname(os.path.abspath(self.clubactivitycachefilename))
            with NamedTemporaryFile(mode='w', suffix='.stravacache', delete=False, dir=cachedir) as tempcache:
                tempclubactivitycachefilename = tempcache.name
                for id in self.clubactivitycache:
                    tempcache.write('{}\n'.format(json.dumps(self.clubactivitycache[id])))

            # now overwrite the previous version of the clubactivitycachefile with the new clubactivitycachefile
            try:
                # atomic operation in Linux
                os.rename(tempclubactivitycachefilename, self.clubactivitycachefilename)

            # should only happen under windows
            except OSError:
                os.remove(self.clubactivitycachefilename)
                os.rename(tempclubactivitycachefilename, self.clubactivitycachefilename)

    #----------------------------------------------------------------------
    def getathleteclubs(self):
    #----------------------------------------------------------------------
        """
        retrieve current athlete's clubs
        """

        # requires python 2.7.9+ for secure ssl
        url = 'https://www.strava.com/api/v3/athlete/clubs'

        # initialize payload
        payload = {'access_token':self.user}
        # payload['per_page'] = perpage
        # payload['page'] = 1

        r = requests.get(url, params=payload)
        r.raise_for_status()

        return r.json()

    #----------------------------------------------------------------------
    def getclubdetails(self,clubid):
    #----------------------------------------------------------------------
        """
        retrieve club information

        :param clubid: strava id for club
        """

        # requires python 2.7.9+ for secure ssl
        url = 'https://www.strava.com/api/v3/clubs/{}'.format(clubid)

        # initialize payload
        payload = {'access_token':self.user}
        # payload['per_page'] = perpage
        # payload['page'] = 1

        r = requests.get(url, params=payload)
        r.raise_for_status()

        return r.json()

    #----------------------------------------------------------------------
    def getclubactivities(self,clubid,before=None,after=None,perpage=200,maxactivities=None,**filters):
    #----------------------------------------------------------------------
        """
        retrieve activities for a club

        :param clubid: strava id for club
        :param before: epoch time activities should be before
        :param after: epoch time activities should be after
        :param perpage: (debug) how many activities per request, max 200 per strava api docs
        :param maxactivities: (debug) max number of activities to return, None means all
        :param filters: additional filters to compare against returned activities {'field1':value, 'field2':[list,of,values]}
        """

        # requires python 2.7.9+ for secure ssl
        url = 'https://www.strava.com/api/v3/clubs/{}/activities'.format(clubid)

        # initialize payload
        if not before:
            before = int(time.time())
        payload = {'access_token':self.user, 'per_page':perpage}
        payload['page'] = 1
        #payload['before'] = before

        # activities are returned in a list, most recent activity first
        # loop getting activities until oldest one is older than 'after' argument
        activities = []
        more = True
        while more:
            r = requests.get(url, params=payload)
            r.raise_for_status()

            theseactivities = r.json()
            if len(theseactivities) > 0:
                earliestactivity = theseactivities[-1]
                earliesttime = stravatime.asc2epoch(earliestactivity[DATEFIELD])

                # are we done?
                if after and earliesttime < after:
                    more = False
                    # get rid of activities at the end which are before 'after'
                    while earliesttime < after:
                        theseactivities.pop()
                        if len(theseactivities) == 0: break
                        earliestactivity = theseactivities[-1]
                        earliesttime = stravatime.asc2epoch(earliestactivity[DATEFIELD])
            
            # collect these activities -- are there any left?
            if len(theseactivities) > 0: 
                activities += theseactivities
                #payload['before'] = earliesttime
                payload['page'] += 1


            # we're done if theseactivities is empty
            else:
                more = False

            # we're done if caller wanted limit on number of activities, and we're at or over the limit
            if maxactivities:
                if len(activities) >= maxactivities:
                    more = False
                    activities = activities[:maxactivities]

        # update activity cache
        for activity in activities:
            id = activity['id']
            if id not in self.clubactivitycache:
                self.clubactivitycache[id] = activity
                self.clubactivitycacheadded += 1
        self.clubactivitycachesize = len(self.clubactivitycache)

        # we're outta here
        return activities

    #----------------------------------------------------------------------
    def getathleteactivities(self,athlete,after=None,perpage=200,maxactivities=None,**filters):
    #----------------------------------------------------------------------
        '''
        This doesn't work. Why does strava not allow viewing other athlete data?
        '''

        # requires python 2.7.9+ for secure ssl
        url = 'https://www.strava.com/api/v3/athletes/{}/activities'.format(athlete)

        # initialize payload
        payload = {'access_token':self.user}
        # payload['per_page'] = perpage
        # payload['page'] = 1

        r = requests.get(url, params=payload)
        r.raise_for_status()

        return r.json()

    #----------------------------------------------------------------------
    def clubactivitycache2csv(self,mapping=None,outfile=None):
    #----------------------------------------------------------------------
        '''
        dump the club activity cache to a csv file

       :param mapping: OrderedDict {'outfield1':'infield1', 'outfield2':outfunction(cacherecord), ...} or ['inoutfield1', 'inoutfield2', ...]
       :param outfile: optional output file
       :rtype: lines from output file
           '''
        # set up default mapping
        if not mapping:
            mapping = OrderedDict([
                ('workout_id',      'id'),
                ('start_date',      'start_date_local'),
                ('name',            lambda rec: '{} {}'.format(unicode2ascii(rec['athlete']['firstname']), unicode2ascii(rec['athlete']['lastname']))),
                ('fname',           lambda rec: '{} {}'.format(unicode2ascii(rec['athlete']['firstname']))),
                ('lname',           lambda rec: '{} {}'.format(unicode2ascii(rec['athlete']['lastname']))),
                ('gender',          lambda rec: '{} {}'.format(unicode2ascii(rec['athlete']['sex']))),
                ('type',            'type'),
                ('workout_type',    lambda rec: workout_type[rec['workout_type']]),
                ('activity_name',   lambda rec: unicode2ascii(rec['name'])),
                ('distance(m)',     'distance'),
                ('time(s)',         'elapsed_time'),
            ])

        activities = list(self.clubactivitycache.values())
        csvrecords = record2csv(activities,mapping,outfile=outfile)
        return csvrecords

#----------------------------------------------------------------------
def updatestravaclubactivitycache(): 
#----------------------------------------------------------------------
    '''
    script to update the strava club activity cache

    usage: updatestravaclubactivitycache [-h] [-v] cachefile clubname

        script to update the strava club activity cache


    positional arguments:
      cachefile      pathname of file in which cache is saved
      clubname       full name of club as known to strava

    optional arguments:
      -h, --help     show this help message and exit
      -v, --version  show program's version number and exit
    '''

    descr = '''
    script to update the strava club activity cache
    '''
    
    parser = argparse.ArgumentParser(description=descr, formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running', version.__version__))
    parser.add_argument('cachefile', help="pathname of file in which cache is saved")
    parser.add_argument('clubname', help="full name of club as known to strava")
    parser.add_argument('--configfile', help='optional configuration filename', default=None)
    args = parser.parse_args()

    # let user know what is going on
    print('Updating Strava club activity cache for "{}"'.format(args.clubname))

    # configuration file supplied -- pull credentials from the app section
    if args.configfile:
        from loutilities.configparser import getitems
        appconfig = getitems(args.configfile, 'app')
        stravakey = appconfig['STRAVAKEY']
    
    # no configuration file, the credentials should be retrieved with loutilities.apikey
    else:
        stravakey = None

    # instantiate the Strava object, which opens the cache
    ss = Strava(args.cachefile, key=stravakey)

    # get the club id
    clubs = ss.getathleteclubs()
    clubid = None
    for club in clubs:
        if club['name'] == args.clubname:
            clubid = club['id']
            break
    
    # error if we didn't find the club
    if not clubid:
        sys.exit('ERROR: club "{}" not found'.format(args.clubname))

    # retrieve all the latest activities
    activities = ss.getclubactivities(clubid)
    numadded = ss.clubactivitycacheadded
    cachesize = ss.clubactivitycachesize

    # close the object, which saves the cache
    ss.close()

    # let user know how we did
    print('   update complete:')
    print('      {} activities received from Strava'.format(len(activities)))
    print('      added {} of these to cache'.format(numadded))
    print('      new cache size = {}'.format(cachesize))

#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    unit test for strava.py
    '''
    
    parser = argparse.ArgumentParser(description=descr, formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running', version.__version__))
    args = parser.parse_args()

    # this would be a good place for unit tests
    
# ##########################################################################################
#	__main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
# ##########################################################################################
#   parserun.py
#       parse run from garmin training center
#
#   Date        Author      Reason
#   ----        ------      ------
#   10/25/09    Lou King    Create
#
# ##########################################################################################

# standard
import optparse
import datetime
import re
import os
import math

# home grown
from running.running import xmldict

MHR = 204               # Lou's MHR.  Change for yours
METERPMILE = 1609.3439941
EPSILON = METERPMILE - 1600 + 1 # make sure 1600 ~= mile

# ##########################################################################################
def timestr(tsm,hralways=False):
#       return time in seconds per mile
# ##########################################################################################
    """
    return time string for time in seconds/mile
    
    :param tsm: time in seconds/mile
    :param hralways: True if the hour portion should be in the string.  False if hour is only present if >= 1 hour.  Default=False
    """
    
    # parse out the integer and microsecond portion.  Add 1/2 second for rounding
    tsec = int(tsm)
    tusec = ((tsm-tsec)*1E6)+.5E6
    
    # add the duration in seconds, microseconds to epoch time of 0, allowing reformat using strftime function
    d = datetime.timedelta (0, tsec, tusec)
    dt1 = datetime.datetime(2009,1,1,0,0,0)
    dt2 = dt1 + d
    
    # if it's greater than an hour or caller requested, then show the hours, else just show minutes:seconds
    if (tsec+(tusec/1E6) > 60*60) or hralways:
        fmt = "%H:%M:%S" 
    else:
        fmt = "%M:%S"
    s = dt2.strftime(fmt)
    
    # get rid of leading 0, if present
    # TBD - needs work for hour=0, for now just skip if hralways
    if not hralways:
        s = re.sub('^0','',s)
    
    return s;

# ##########################################################################################
def pace (
#       return pace in seconds per mile
    ts, 
#       time in seconds
    d):
#       distance in meters
# ##########################################################################################

    # calculate time in seconds per mile
    tsm = ts * (METERPMILE / d)
    
    return timestr(tsm)

#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    usage = "usage: %prog [options] <tcxfile> <laplist>\n\n"
    usage += "where:\n"
    usage += "  <tcxfile>\tfile output from Garmin Training Center\n"
    usage += "  <laplist> is one of two formats:\n"
    usage += "     no switches\tsets of laps to be averaged, i.e., 5,3 means average first 5 laps, next 3 laps, then remaining\n"
    usage += "     -interval\tnum laps before first workout then num repeats\n"

    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-i", "--interval", dest="interval", action="store_true", help="treat input as intervals: num laps before first workout then num repeats")
    parser.set_defaults(interval=False)
    (options, args) = parser.parse_args()

    tcxfile = args.pop(0)
    laplist = args.pop(0)
    sets = [int(si) for si in laplist.split(',')]
    numsets = len(sets)

    root = xmldict.readXmlFile(tcxfile)
    laps = root['TrainingCenterDatabase']['Activities']['Activity']['Lap']

    numlaps = len(laps)

    # set up uselap (ul) array
    ul = [True for i in range(numlaps)]

    # if interval (--interval) switch is set, reset to use a single set, and update uselap (ul) array
    # to force accumulation of only the indicated repeats
    if (options.interval):
        ul = [False for i in range(numlaps)]
        (numbefore,numint) = sets
        for i in range(numint):
            ul[numbefore+i*2] = True

        sets = [numlaps,]
        numsets = 0


    more = True
    lp = 0
    tttime = 0
    tthrtime = 0
    tthr = 0
    ttdist = 0

    setstr = ""
    csvstr = "distance,time,pace,avghr\n"
    pacesstr = {}
    pacesoneline = ''
    pacesexcel = '=('
    hrsexcel = '=('
    firstul = True

    for s in range(numsets+1):  # and more
        ttime = 0
        thrtime = 0
        thr = 0
        tdist = 0
        if s < numsets:
            setlimit = sets[s]
        else:
            setlimit = numlaps # numlaps for convenient ceiling for limit
        for i in range(setlimit):   # and more; i++) {
            time = float(laps[lp]['TotalTimeSeconds'])
            dist = float(laps[lp]['DistanceMeters'])        # bug in xmldict?  Seems to put into a list if any child tag has the same name
    		# hr is optional
            hr = int(laps[lp]['AverageHeartRateBpm']['Value']) if 'AverageHeartRateBpm' in laps[lp] else 0
            
            if dist != 0.0:
            
                # accumulate for this set
                # for interval switch, only indicated laps are averaged
                if (ul[lp]):
                    ttime += time
                    if (lp!=0):
                        thrtime += time 
                        thr += hr * time
                    
                    tdist += dist
                    if not firstul:
                        pacesoneline += ', '
                        pacesexcel += '+'
                        hrsexcel += '+'
                    firstul = False
                
                
                # accumulate for whole run
                tttime += time
                if (lp!=0):
                    tthrtime += time
                    tthr += hr * time
                
                ttdist += dist
                
                pacesstr[lp] = pacesstr.setdefault(lp,"") + timestr(time) + "({0:d})".format(hr)
                csvstr += "{0:.2f},{1},{2},{3}\n".format(dist/METERPMILE,timestr(time),pace(time,dist),hr)
                if (ul[lp]):
                    pacesoneline += timestr(time) + "({0:d})".format(hr)
                    pacesexcel += '"{0}"'.format(timestr(time,True))
                    hrsexcel += '{0:d}'.format(hr)

                # indicate distance=pace for odd-distanced splits
                if (dist != METERPMILE):
                    pacesstr[lp] += " [{0:.2f}={1}]".format(dist/METERPMILE,pace(time,dist))

            # weird 0 distance lap
            else:
                pacesstr[lp] = pacesstr.setdefault(lp,"") + timestr(time) + "({0:d})".format(hr)
                
            lp += 1
            if (lp == numlaps):
                more = False
                break
        
        # calculate averages over this set
        if thrtime > 0:
            ahr = int ((thr / thrtime) + 0.5)
        else:
            ahr = "n/a"
        apace = pace (ttime, tdist)
        adist = tdist / METERPMILE
        if s < numsets:
            intmiles = int(adist+0.5)
            if math.fabs(tdist/intmiles-METERPMILE) < EPSILON:
                adistpr = int(adist+EPSILON/METERPMILE)
            else:
                adistpr = adist 
        else: 
            adistpr = "{0:.2f}".format(adist)
        
        if (s > 0):
            setstr += ", "
        setstr += "{0}@{1}({2})".format(adistpr,apace,ahr)

        if not more:
            break
        

    # calculate averages over the whole run
    athr = int ((tthr / tthrtime) + 0.5)
    atpace = pace (tttime, ttdist)
    atdist = ttdist / METERPMILE
    totstr = "{0:.1f} miles, {1}, {2}/mi, AHR {3} ({4}% MHR)".format (atdist, timestr(tttime), atpace, athr, int((athr*100/MHR)+0.5))

    CSV = open ("history.csv","w")
    CSV.write (csvstr)
    CSV.close()

    OUT = open ("temp.txt","w")
    OUT.write ("{0}\n".format(totstr))
    OUT.write ("{0}\n\n".format(setstr))
    if (options.interval):
        countlaps = 0
        OUT.write ("splits:\n")
        for lp in range(numlaps):
            if (ul[lp]):
                OUT.write ("{0}\n".format(pacesstr[lp]))
                countlaps += 1
        OUT.write ("\n")
    else:
        countlaps = numlaps
        

    OUT.write ("all splits:\n")
    for lp in range(numlaps):
        split = lp+1
        OUT.write ("{0} - {1}\n".format(split,pacesstr[lp]))

    OUT.write('\n')
    OUT.write(pacesoneline+'\n')
    OUT.write(pacesexcel+')\n')
    OUT.write(hrsexcel+')/'+str(countlaps)+'\n')

    OUT.close()
    os.startfile ("temp.txt")

# ##########################################################################################
#   __main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
###########################################################################################
#   athlinks - access methods for athlinks.com
#
#   Date        Author      Reason
#   ----        ------      ------
#   11/12/13    Lou King    Create
#
#   Copyright 2013 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
athlinks - access methods for athlinks.com
===================================================
'''

# standard
import pdb
import argparse
import os.path
import urllib.request, urllib.parse, urllib.error
import json
import time
import logging
logging.basicConfig(format='%(asctime)s %(levelname)s:%(message)s')

# pypi
import httplib2

# github

# other

# home grown
from loutilities import apikey
from running import accessError, parameterError

# access stuff
ATHLINKS_URL = 'https://api.athlinks.com'
RESULTS_SEARCH = 'Results/search'
RACE_SEARCH = 'races/{id}'
COURSE_SEARCH = 'races/{raceid}/{courseid}'
MEMBER_RESULTS_SEARCH = 'athletes/results/{id}'
MEMBER_DETAILS = 'athletes/details/{handle}'
ATH_CAT_RUNNING = 2

HTTPTIMEOUT = 60
SERVERDELAY = 20
MPERMILE = 1609.344


#----------------------------------------------------------------------
def gettime(athlinkstime):
#----------------------------------------------------------------------
    '''
    return epoch time based on athlinks time
    
    :param athlinkstime: time from athlinks record
    :rtype: int containing epoch time
    '''
    # parse racedate.  racedate is '\Date(<epochtime*1000>)\'  
    while athlinkstime[0] not in '0123456789':
        athlinkstime = athlinkstime[1:]
    while athlinkstime[-1] not in '0123456789':
        athlinkstime = athlinkstime[:-1]
    return int(athlinkstime)/1000  # strange format, but that's what it is

#----------------------------------------------------------------------
def dist2miles(distunit,disttypeid):
#----------------------------------------------------------------------
    '''
    get distance in miles, based on athlinks distunit, disttypeid
    
    :param distunit: number of units of distance, unit based on disttypeid
    :param disttypeid: meters=6, others unknown
    '''
    if disttypeid != 6:
        raise parameterError('unknown disttypeid {}'.format(disttypeid))
    return distunit / MPERMILE

#----------------------------------------------------------------------
def dist2km(distunit,disttypeid):
#----------------------------------------------------------------------
    '''
    get distance in kilometers, based on athlinks distunit, disttypeid
    
    :param distunit: number of units of distance, unit based on disttypeid
    :param disttypeid: meters=6, others unknown
    '''
    if disttypeid != 6:
        raise parameterError('unknown disttypeid {}'.format(disttypeid))
    return distunit / 1000

########################################################################
class Athlinks():
########################################################################
    '''
    access methods for athlinks.com
    '''

    #----------------------------------------------------------------------
    def __init__(self, key=None, debug=False):
    #----------------------------------------------------------------------
        """
        initialize http and get athlinks key
        """
        # get credentials from configuration
        self.key = key
        if not self.key:
            ak = apikey.ApiKey('Lou King','running')
            try:
                self.key = ak.getkey('athlinks')
            except apikey.unknownKey:
                raise parameterError("'athlinks' key needs to be configured using apikey")
        
        # need http object
        self.http = httplib2.Http(timeout=HTTPTIMEOUT)

        # set up logging level
        self.log = logging.getLogger('running.athlinks')
        self.setdebug(debug)
        
        # count how many pages have been retrieved
        self.urlcount = 0
        
    #----------------------------------------------------------------------
    def setdebug(self,debugval):
    #----------------------------------------------------------------------
        '''
        set debugging attribute for this class
        
        :param debugval: set to True to enable debugging
        '''
        if not debugval:
            level = logging.INFO
        else:
            level = logging.DEBUG
        self.log.setLevel(level)
        
    #----------------------------------------------------------------------
    def geturlcount(self):
    #----------------------------------------------------------------------
        '''
        each time a url is retrieved, this counter is bumped
        
        :rtype: integer, number of url's retrieved
        '''
        return self.urlcount

    #----------------------------------------------------------------------
    def listathleteresults(self,name,**filt):
    #----------------------------------------------------------------------
        """
        return results which match an athlete's name
        
        :param name: name of athlete
        :param **filt: keyword parameters to filter with
        :rtype: list of athlinks Race dicts
        """
        
        # get the data for this athlete
        page = 1
        races = []
        while True:
            data = self._get(RESULTS_SEARCH,
                               pagesize=500,
                               page=page,
                               name=name,
                               includeclaimed=True
                               )
            if len(data['List']) == 0: break
            
            races += data['List']
            page += 1

        def _checkfilter(checkdict):
            for key in filt:
                if key not in checkdict or checkdict[key] != filt[key]:
                    return False
            return True

        races = list(filter(_checkfilter,races))
        return races
        
    #----------------------------------------------------------------------
    def getmember(self,handle):
    #----------------------------------------------------------------------
        '''
        get member record associated with handle
        
        :param handle: handle for member, ID or email should work
        '''
        handle = urllib.parse.quote(str(handle))
        data = self._get(MEMBER_DETAILS.format(handle=handle)
                           )
        return data
        
    #----------------------------------------------------------------------
    def getrace(self,id):
    #----------------------------------------------------------------------
        '''
        get race record associated with id
        
        :param id: id of race
        '''
        data = self._get(RACE_SEARCH.format(id=id)
                           )
        return data
        
    #----------------------------------------------------------------------
    def getcourse(self,raceid,courseid):
    #----------------------------------------------------------------------
        '''
        get race record associated with id
        
        :param id: id of race
        '''
        data = self._get(COURSE_SEARCH.format(raceid=raceid,courseid=courseid)
                           )
        return data
        
    #----------------------------------------------------------------------
    def _get(self,method,**params):
    #----------------------------------------------------------------------
        """
        get method for athlinks access
        
        :param method: athlinks method to call
        :param **params: parameters for the method
        """
        
        params['format'] = 'json'
        params['key'] = self.key
        
        body = urllib.parse.urlencode(params)
        url = '{}/{}?{}'.format(ATHLINKS_URL,method,body)
        
        # loop RETRIES times for timeout or other error
        retries = 20
        while retries > 0:
            retries -= 1
            try:
                self.log.debug(url)
                resp,jsoncontent = self.http.request(url)

                if resp.status != 200:
                    raise accessError('URL response status = {0}'.format(resp.status))
                
                # unmarshall the response content
                content = json.loads(jsoncontent)

                self.urlcount += 1
                break

            except Exception as e:
                if retries == 0:
                    self.log.info('{} requests attempted'.format(self.geturlcount()))
                    self.log.error('http request failure, retries exceeded: {0}'.format(e))
                    if repr(e) == 'ValueError: No JSON object could be decoded':
                        self.log.error('   jsoncontent={}'.format(jsoncontent))
                    raise
                self.log.warning('http request failure: {0}'.format(e))
                time.sleep(SERVERDELAY)  # give the server some time to recover

        return content 
        
#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    unit test for athlinks.py
    '''
    
    parser = argparse.ArgumentParser(description=descr,formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running',version.__version__))
    args = parser.parse_args()

    # this would be a good place for unit tests
    
# ##########################################################################################
#	__main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
# ###############################################################################
# getmembers-competitor - get members who ran competitor race (e.g., Rock 'n' Roll Marathon)
#
# Author: L King
#
# REVISION HISTORY:
#   03/17/15    L King      Create
# ###############################################################################
'''
getmembers-competitor - get members who ran competitor race (e.g., Rock 'n' Roll Marathon)
==============================================================================================

Usage::

    TBA
'''

# standard libraries
import csv
import argparse
from string import maketrans
from collections import OrderedDict

# home grown libraries
from running.running import version
from .competitor import Competitor

# convert competitorResult to expected format for runningclub.RaceResults
competitor2raceresult = OrderedDict([
    ('oaplace',     'place'),
    ('name',        'name'),
    ('genplace',    'gender place',),
    ('divplace',    'division place',),
    ('age',         'age'),
    ('gender',      'gender'),
    ('hometown',    'hometown'),
    ('racetime',    'time'),
    ('racedate',    'race date'),
    ('raceloc',     'race location'),
    ('racename',    'race name'),
    ('distmiles',   'miles'),
    ('distkm',      'km')
])

#-------------------------------------------------------------------------------
def getcompetitorrace(outfile,eventid,eventinstanceid,singleeventid,limit=None):
#-------------------------------------------------------------------------------
    '''
    create file for competitor race based on eventid, eventinstanceid, singleeventid
    
    :param outfile: base filename for output file, event and distance are appended to name
    :param eventid: event id from running.competitor.com
    :param eventinstanceid: event instance id from running.competitor.com
    :param singleeventid: single event id from running.competitor.com
    :param limit: limit number of records (for testing only)
    
    :rtype: name of file which was created
    '''

    cc = Competitor()
    cc.setraceyear(eventid,eventinstanceid,singleeventid)
    results = cc.getresults(limit)
    
    # build filename using racename and distance
    # racename needs to be str() else translate gets ValueError because transtab is not Unicode mapping
    racename = str(results[0].racename) 
    dist = '{:.1f}'.format(results[0].distmiles)
    if dist == 13.1:
        dist = 'halfmarathon'
    elif dist == 26.2:
        dist = 'marathon'
    else:
        dist = '{}miles'.format(dist)
    # change all ' ' to '-' and quotes to 'x' in racename
    transtab = maketrans(''' '"''','-xx')   
    tracename = racename.translate(transtab)
    outfilename = '{}-{}-{}.csv'.format(outfile,tracename,dist)
    
    # open output results file
    RS_ = open(outfilename,'w',newline='')
    RS = csv.DictWriter(RS_,list(competitor2raceresult.values()))
    RS.writeheader()
    
    # write the results to the file
    for result in results:
        filerow = {}
        for attr in result.attrs:
            filerow[competitor2raceresult[attr]] = getattr(result,attr)
        RS.writerow(filerow)

    # close file
    RS_.close()
    
    # let caller know the name of the file which was built
    return outfilename
    
#-------------------------------------------------------------------------------
def getcompetitor(outfile):
#-------------------------------------------------------------------------------
    '''
    put competitor results into csv file per race
    
    :param outfile: base of output file name for competitor files
    
    :rtype: list containing names of files which were created
    '''
    
    # build list of files which are output
    resultfiles = []
    
    # build half marathon file
    thisfile = getcompetitorrace(outfile,54,227,797)
    resultfiles.append(thisfile)

    # build marathon file
    thisfile = getcompetitorrace(outfile,54,227,791)
    resultfiles.append(thisfile)
    
    # let caller know what files were built
    return resultfiles

#-------------------------------------------------------------------------------
def main():
#-------------------------------------------------------------------------------

    parser = argparse.ArgumentParser(version='running {0}'.format(version.__version__))
    parser.add_argument('outfile', help="base name for output file")
    parser.add_argument('-e','--eventid', help="eventid for competitor.com",type=int,default=None)
    parser.add_argument('-i','--instanceid', help="event instance for competitor.com (which year)",type=int,default=None)
    parser.add_argument('-s','--singleeventid', help="single eventid for competitor.com (which distance)",type=int,default=None)
    parser.add_argument('-l','--limit', help="limit number of records (for testing only)",type=int,default=None)
    args = parser.parse_args()
    
    # get arguments
    outfile = args.outfile
    eventid = args.eventid
    instanceid = args.instanceid
    singleeventid = args.singleeventid
    limit = args.limit
    
    # only collect information from competitor.com if output file is defined
    resultfile = getcompetitorrace(outfile,eventid,instanceid,singleeventid,limit)
    print(('generated {}'.format(resultfile)))
        
# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




#!/usr/bin/python
# ###############################################################################
# analyzeagegrade - analyze age grade race data
#
# Author: L King
#
# REVISION HISTORY:
#   08/08/12    L King      Create
# ###############################################################################
"""
analyzeagegrade - analyze age grade race data
=====================================================================================

Usage::

    TBA
"""

# standard libraries
import csv
import argparse
import time
import logging
logging.basicConfig(format='%(asctime)s %(levelname)s: %(message)s')
log = logging.getLogger('running.analyzeagegrade')
log.setLevel(logging.INFO)

# other libraries
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as colors
import matplotlib.dates as mdates
import matplotlib.font_manager as fontmgr
from scipy import stats

# home grown libraries
from running.running import version
from loutilities import timeu
from runningclub import agegrade
import running.running.runningahead as runningahead
from running.running.runningahead import FIELD

class unexpectedEOF(Exception): pass
class invalidParameter(Exception): pass

METERSPERMILE = 1609.344
MAXMETER = 4999
SUBS = {1609:'1M',3219:'2M',4989:'5K',5000:'5K',8047:'5M',10000:'10K',15000:'15K',
        16093:'10M',21082:'HM',21097:'HM',42165:'Marathon',42195:'Marathon',
        80467:'50M',160934:'100M'} #

tdisp = timeu.asctime('%m/%d/%Y')
# pull in age grade object
ag = agegrade.AgeGrade()
    

#-------------------------------------------------------------------------------
def distmap(dist):
#-------------------------------------------------------------------------------
    """
    map distance to display metric
    
    :param dist: distance to map
    :rtype: float display metric for distance
    """
    return dist/100
    
########################################################################
class AgeGradeStat():
########################################################################
    '''
    statistic for age grade analysis, for a single runner
    
    :param date: date in datetime format
    :param dist: distance in meters
    :param time: time in seconds
    :param ag: age grade percentage (float, 0-100)
    :param race: race name
    :param loc: location of race
    :param source: source of data
    :param fuzzyage: 'Y' if age check was done based on age group rather than exact age, None otherwise
    :param priority: priority for deduplication, lowest value is kept (lower number = higher priority)
    '''
    attrs = 'race,date,loc,dist,time,ag,source,fuzzyage,priority'.split(',')
    
    #-------------------------------------------------------------------------------
    def __init__(self,date=None,dist=None,time=None,ag=None,race=None,loc=None,source=None,fuzzyage=None,priority=1):
    #-------------------------------------------------------------------------------
        self.date = date
        self.dist = dist
        self.time = time
        self.ag = ag
        self.race = race
        self.loc = loc
        self.source = source
        self.fuzzyage = fuzzyage
        self.priority = priority
        
    #-------------------------------------------------------------------------------
    def __repr__(self):
    #-------------------------------------------------------------------------------
        retval = '{}({}, {} meters, {} secs'.format(self.__class__,tdisp.dt2asc(self.date),self.dist,self.time)
        if self.ag:
            retval += ', age grade = {}'.format(self.ag)
        if self.race:
            retval += ', {}'.format(self.race)
        retval += ')'
        return retval

########################################################################
class TrendLine():
########################################################################
    '''
    regression line parameters (ref http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.linregress.html)
    
    :param slope: slope of the regression line
    :param intercept: intercept fo the regression line
    :param rvalue: correlation coefficient
    :param pvalue: two-sided p-value for hypothesis test whose null hypothesis is that the slope is zero
    :param stderr: standard error of the estimate
    '''
    
    #-------------------------------------------------------------------------------
    def __init__(self,slope,intercept,rvalue,pvalue,stderr):
    #-------------------------------------------------------------------------------
        self.slope = slope
        self.intercept = intercept
        self.rvalue = rvalue
        self.pvalue = pvalue
        self.stderr = stderr
        
    #-------------------------------------------------------------------------------
    def __repr__(self):
    #-------------------------------------------------------------------------------
        retval = 'analyzeagegrade.TrendLine(slope {:0.2f}, intercept {:0.2f}, rvalue {:0.2f}, pvalue {:0.2f}, stderr {:0.2f})'.format(
            self.slope, self.intercept, self.rvalue, self.pvalue,self.stderr)
        return retval
    
        
########################################################################
class AnalyzeAgeGrade():
########################################################################
    '''
    age grade analysis
    '''
    
    #-------------------------------------------------------------------------------
    def __init__(self, size=False):
    #-------------------------------------------------------------------------------
        self.exectime = time.time()
        self.gender = None
        self.dob = None
        self.cmapsm = None
        self.renderfname = '{who}-ag-analysis-{date}-{time}.png'
        self.xlim = {'left':None,'right':None}
        self.ylim = None    # TODO: make 'top','bottom' dict
        
        # sized markers or just color
        self.size = size
        if size:
            self.s_size = 'size'
        else:
            self.s_size = 'color'
        
        self.clear()
        
    #-------------------------------------------------------------------------------
    def clear(self):
    #-------------------------------------------------------------------------------
        '''
        clear statistics
        '''
        # stats = list(AgeGradeStat(),... ) 
        self.stats = []
        
        # self.dists = set of distances included in stats, rounded
        self.dists = set([])

    #-------------------------------------------------------------------------------
    def add_stat(self, date, dist, time, **kwargs):
    #-------------------------------------------------------------------------------
        '''
        add an individual statistic
        
        :param date: date in datetime format
        :param dist: distance in meters
        :param time: time in seconds
        :param kwargs: keyword arguments, must match AgeGradeState attrs
        '''
        
        self.stats.append(AgeGradeStat(date,dist,time,**kwargs))
        self.dists.add(round(dist))
        
    #-------------------------------------------------------------------------------
    def del_stat(self, stat):
    #-------------------------------------------------------------------------------
        '''
        delete the indicated statistic
        
        :param stat: :class:`AgeGradeStat` to delete
        '''
        try:
            self.stats.remove(stat)
        except ValueError:
            log.warning('del_stat: failed to delete {}'.format(stat))
        
    #-------------------------------------------------------------------------------
    def get_stats(self):
    #-------------------------------------------------------------------------------
        '''
        return stats collected
        
        :rtype: list of :class:`AgeGradeStat` entries
        '''
        return self.stats
    
    #-------------------------------------------------------------------------------
    def deduplicate(self):
    #-------------------------------------------------------------------------------
        '''
        remove statistics which are duplicates, assuming stats on same day
        for same distance are duplicated
        '''
        
        ## sort statistics by (date,dist)
        #decstats = [((s.date,s.dist,s.priority),s) for s in self.stats]
        #decstats.sort()
        #
        ## collect unique statistics, within epsilon distance
        #EPS = .1   # epsilon -- if event distance is within this tolerance, it is considered the same
        #
        #key,firststat = decstats.pop(0)
        #deduped = [firststat]
        #for key,thisstat in decstats:
        #    laststat = deduped[-1]
        #    # only accumulate if date or distance mismatch from last collected
        #    if laststat.date != thisstat.date or abs((laststat.dist-thisstat.dist)/laststat.dist) > EPS:
        #        deduped.append(thisstat)
        #
        #dupremoved = len(self.stats) - len(deduped)
        #if dupremoved > 0:
        #    log.debug('{} duplicate points removed, runner {}'.format(dupremoved,self.who))
        
        # be careful of degenerate case
        if len(self.stats) == 0:
            return

        # collect unique statistics, within epsilon distance
        EPS = .1   # epsilon -- if event distance is within this tolerance, it is considered the same

        # sort self.stats into stats, by date,distance
        decstats = sorted([((s.date,s.dist),s) for s in self.stats])
        stats = [ds[1] for ds in decstats]
        
        # deduplicate stats, paying attention to priority when races determined to be the same
        deduped = []
        while len(stats) > 0:
            # get the first entry, that's the first "samerace"
            thisstat = stats.pop(0)
            sameraces = [(thisstat.priority,thisstat)]
            
            # pull races off stats when the race date and distance are the same
            # distance has to be within epsilon to be deduced to be the same
            while   len(stats) > 0 \
                    and thisstat.date == stats[0].date \
                    and abs((thisstat.dist - stats[0].dist) / thisstat.dist) <= EPS:
                stat = stats.pop(0)
                sameraces.append((stat.priority,stat))
            
            # sort same races by priority, and add highes priority (lowest valued) to deduped list
            sameraces.sort()
            prio,stat = sameraces[0]
            deduped.append(stat)
        
        dupremoved = len(self.stats) - len(deduped)
        if dupremoved > 0:
            log.debug('{} duplicate points removed, runner {}'.format(dupremoved,self.who))

        # replace self.stats with deduplicated version
        self.stats = deduped

    #-------------------------------------------------------------------------------
    def set_renderfname(self,renderfname):
    #-------------------------------------------------------------------------------
        '''
        set filename template for rendered files
        
        form is similar to '{who}-ag-analysis-{date}-{time}.png'
        
        where:
        
            * who - comes from :meth:`set_runner` who parameter
            * date - comes from the time the :class:`AnalyzeAgeGrade` object was created, yyyy-mm-dd
            * time - comes from the time the :class:`AnalyzeAgeGrade` object was created, hhmm
        
        :param renderfname: filename template
        '''
        self.renderfname = renderfname
        
    #-------------------------------------------------------------------------------
    def get_outfilename(self):
    #-------------------------------------------------------------------------------
        '''
        get output filename
        
        must be called after :meth:`set_runner` and :meth:`set_renderfname`
        
        :rtype: name of output file for plot
        '''
        tdate = timeu.asctime('%Y-%m-%d')
        ttime = timeu.asctime('%H%M')
        outfilename = self.renderfname.format(who=self.who,date=tdate.epoch2asc(self.exectime),time=ttime.epoch2asc(self.exectime))
        return outfilename
    
    #-------------------------------------------------------------------------------
    def set_runner(self,who,gender=None,dob=None):
    #-------------------------------------------------------------------------------
        '''
        set runner parameters required for age grade analysis
        
        :param who: name of runner
        :param gender: M or F
        :param dob: datetime date of birth
        '''
        self.who = who
        self.gender = gender
        self.dob = dob
        
    #-------------------------------------------------------------------------------
    def get_runner(self):
    #-------------------------------------------------------------------------------
        '''
        return runner data
        
        :rtype: name,gender,dob
        '''
        return self.who,self.gender,self.dob
    
    #-------------------------------------------------------------------------------
    def set_xlim(self,left,right):
    #-------------------------------------------------------------------------------
        '''
        set x limits
        
        :param left: datetime value of left limit for x
        :param right: datetime value of right limit for x
        '''
        #self.xlim = (left,right)
        self.xlim = {'left':None,'right':None}
        if left:
            self.xlim['left'] = left
        if right:
            self.xlim['right'] = right
        
    #-------------------------------------------------------------------------------
    def set_ylim(self,bottom,top):
    #-------------------------------------------------------------------------------
        '''
        set y limits
        
        :param bottom: value of bottom limit for y
        :param top: value of top limit for y
        '''
        self.ylim = (bottom,top)
        
    #-------------------------------------------------------------------------------
    def set_colormap(self,dists=None):
    #-------------------------------------------------------------------------------
        '''
        set color mapping for rendering, based on range of distance statistics
        
        :param dists: sequence containing range which must be met by colormap, defaults to stored statistics, meters
        '''
        # set up color normalization
        cnorm = colors.LogNorm()
        if dists:
            cnorm.autoscale(dists)
        else:
            cnorm.autoscale([s.dist for s in self.stats])
        cmap = cm.jet
        self.cmapsm = cm.ScalarMappable(cmap=cmap,norm=cnorm)
        
    #-------------------------------------------------------------------------------
    def getdatafromfile(self, agfile):
    #-------------------------------------------------------------------------------
        '''
        plot the data in dists
        
        :param agfile: name of csv file containing age grade data
        :rtype: 
        '''
        
        _IN = open(agfile,newline='')
        IN = csv.DictReader(_IN,dialect='excel')
    
        # collect data
        linenum = 0
        while True:
            try:
                inrow = next(IN)
                linenum += 1
            except StopIteration:
                break
                
            s_date = inrow['Date']
            date = tdisp.asc2dt(s_date)
            
            dist = float(inrow['Distance (miles)']) * METERSPERMILE
            
            # calculate number of seconds in string field [[hh:]mm:]ss[.000]
            s_rtime = inrow['Net']
            timefields = iter(s_rtime.split(':'))
            rtime = 0.0
            thisunit = float(next(timefields))
            while True:
                rtime += thisunit
                try:
                    thisunit = float(next(timefields))
                except StopIteration:
                    break
                rtime *= 60 # doesn't happen if last field was processed before
            
    
            # age grade calculation was moved to crunch() to crunch age grade and pace
            # this just saves what was in the file in case I ever want to compare
            s_ag = inrow['AG']
            if s_ag:    
                if s_ag[-1] == '%':
                    ag = float(s_ag[:-1])
                else:
                    ag = float(s_ag)
            # we don't care about this entry if AG wasn't captured
            else:
                ag = None
                
            self.dists.add(round(dist))      # keep track of distances to nearest meter
            self.stats.append(AgeGradeStat(date,dist,rtime))
            #print(s_date,date,dist,ag)
            
        _IN.close()
    
    #-------------------------------------------------------------------------------
    def getdatafromra(self):
    #-------------------------------------------------------------------------------
        '''
        get the user's data from RunningAHEAD
        
        :rtype: dists,stats,dob,gender where dists =  set of distances included in stats, stats = {'date':[datetime of race,...], 'dist':[distance(meters),...], 'time':[racetime(seconds),...]}, dob = date of birth (datetime), gender = 'M'|'F'
        '''
        # set up RunningAhead object and get users we're allowed to look at
        ra = runningahead.RunningAhead()    
        users = ra.listusers()
        day = timeu.asctime('%Y-%m-%d') # date format in RunningAhead workout object
        
        # find correct user, grab their workouts
        workouts = None
        for user in users:
            thisuser = ra.getuser(user['token'])
            if 'givenName' not in thisuser: continue    # we need to know the name
            givenName = thisuser['givenName'] if 'givenName' in thisuser else ''
            familyName = thisuser['familyName'] if 'familyName' in thisuser else ''
            thisusername = ' '.join([givenName,familyName])
            if thisusername != self.who: continue            # not this user, keep looking
            
            # grab user's date of birth and gender, if not already supplied
            if not self.dob:
                self.dob = day.asc2dt(thisuser['birthDate'])
            if not self.gender:
                self.gender = 'M' if thisuser['gender']=='male' else 'F'
            
            # if we're here, found the right user, now let's look at the workouts
            firstdate = day.asc2dt('1980-01-01')
            lastdate = day.asc2dt('2199-12-31')
            workouts = ra.listworkouts(user['token'],begindate=firstdate,enddate=lastdate,getfields=list(FIELD['workout'].keys()))
    
            # we've found the right user and collected their data, so we're done
            break
            
        # save race workouts, if any found
        if workouts:
            tempstats = []
            for wo in workouts:
                if wo['workoutName'].lower() != 'race': continue
                thisdate = day.asc2dt(wo['date'])
                thisdist = runningahead.dist2meters(wo['details']['distance'])
                thistime = wo['details']['duration']
                
                tempstats.append((thisdate,AgeGradeStat(thisdate,thisdist,thistime)))
                
        # these may come sorted already, but just in case
        #tempstats.sort()
        
        # put the stats in the right format
        for thisdate,thisstat in tempstats:
            self.stats.append(thisstat)
            self.dists.add(round(thisstat.dist))      # keep track of distances to nearest meter
    
    #-------------------------------------------------------------------------------
    def crunch(self):
    #-------------------------------------------------------------------------------
        '''
        crunch the race data to put the age grade data into the stats
        
        '''
        ### DEBUG>
        debug = False
        if debug:
            tim = timeu.asctime('%Y-%m-%d-%H%M')
            _DEB = open('analyzeagegrade-debug-{}-crunch-{}.csv'.format(tim.epoch2asc(self.exectime,self.who)),'w',newline='')
            fields = ['date','dist','time','ag']
            DEB = csv.DictWriter(_DEB,fields)
            DEB.writeheader()
        ### <DEBUG
            
        # calculate age grade for each sample    
        for i in range(len(self.stats)):
            racedate = self.stats[i].date
            agegradeage = racedate.year - self.dob.year - int((racedate.month, racedate.day) < (self.dob.month, self.dob.day))
            distmiles = self.stats[i].dist/METERSPERMILE
            agpercentage,agtime,agfactor = ag.agegrade(agegradeage,self.gender,distmiles,self.stats[i].time)
            self.stats[i].ag = agpercentage
            
            ### DEBUG>
            if debug:
                thisstat = {}
                for field in fields:
                    thisstat[field] = getattr(self.stats[i],field)
                DEB.writerow(thisstat)
            ### <DEBUG
            
        ### DEBUG>
        if debug:
            _DEB.close()
        ### <DEBUG
    
    #-------------------------------------------------------------------------------
    def render_stats(self,fig):
    #-------------------------------------------------------------------------------
        '''
        plot the data in dists
        
        :param size: true if size needed
        '''
        DEFAULTSIZE = 60
        
        ### DEBUG>
        debug = False
        if debug:
            tim = timeu.asctime('%Y-%m-%d-%H-%M')
            _DEB = open('analyzeagegrade-debug-{}-render.csv'.format(tim.epoch2asc(self.exectime)),'w',newline='')
            fields = ['date','dist','ag','color','label']
            DEB = csv.DictWriter(_DEB,fields)
            DEB.writeheader()
        ### <DEBUG
    
        # make hashed scatter lists
        hdate = {}
        hag = {}
        hsize = {}
        for thisd in self.dists:
            hdate[thisd] = []
            hag[thisd] = []
            hsize[thisd] = []
        for i in range(len(self.stats)):
            d = round(self.stats[i].dist)
            hdate[d].append(self.stats[i].date)
            hag[d].append(self.stats[i].ag)
            if self.size:
                hsize[d].append(distmap(d))
    #            hsize[d].append(self.stats['size'][i])
            else:
                hsize[d].append(DEFAULTSIZE)
        
        # create figure and axes
        fig.autofmt_xdate()
        ax = fig.get_axes()[0]  # only one axes instance
        ax.set_ylabel('age grade percentage')
        #ax.fmt_xdata = mdates.DateFormatter('%Y-%m-%d') # dead?
        fig.suptitle("{}".format(self.who))
            
        lines = []
        labs = []
        l_dists = sorted(self.dists)
        fig.subplots_adjust(bottom=0.1, right=0.85, top=0.93)
        ax.grid(b=True)
        for thisd in l_dists:
            # some results for this distance may have been pulled out due to filtering
            if len(hag[thisd]) == 0: continue
            
            if int(thisd) in SUBS:
                lab = SUBS[int(thisd)]
            else:
                if thisd <=MAXMETER:
                    lab = '{0}m'.format(int(thisd))
                else:
                    lab = '{0:.1f}K'.format(thisd/1000)
            labs.append(lab)
            color = self.cmapsm.to_rgba(thisd)
            numels = len(hdate[thisd])
            line = ax.scatter(hdate[thisd],hag[thisd],s=hsize[thisd],c=[color for i in range(numels)],label=lab,linewidth=.5)
            #lines.append(line)
    
            ### DEBUG>
            if debug:
                thisstat = {}
                for i in range(len(hdate[thisd])):
                    thisstat['date'] = hdate[thisd][i]
                    thisstat['ag'] = hag[thisd][i]
                    thisstat['dist'] = thisd
                    thisstat['label'] = lab
                    thisstat['color'] = self.cmapsm.to_rgba(thisd)
                    DEB.writerow(thisstat)
            ### <DEBUG
    
        # set x (date) label format
        hfmt = mdates.DateFormatter('%m/%d/%y')
        ax.xaxis.set_major_formatter(hfmt)
        ax.xaxis.set_minor_formatter(hfmt)
        labels = ax.get_xticklabels()
        for label in labels:
            label.set_rotation(65)
            label.set_size('xx-small')
    
        # maybe user wants to set xlim
        ax.set_xlim(left=self.xlim['left'],right=self.xlim['right'])
            
        # maybe user wants to set ylim
        # check to see if any points are outside this limit, and print warning
        if self.ylim:
            ax.set_ylim(self.ylim)
            outsidelimits = 0
            numpoints = 0
            for thisd in l_dists:
                for i in range(len(hdate[thisd])):
                    numpoints += 1
                    if hag[thisd][i] < self.ylim[0] or hag[thisd][i] > self.ylim[1]:
                        outsidelimits += 1
            if outsidelimits > 0:
                log.warning('{} of {} points found outside of ylim {}, runner {}'.format(outsidelimits,numpoints,self.ylim,self.who))
        
        # annotate
        
        ### DEBUG>
        if debug:
            _DEB.close()
        ### <DEBUG
    
    
    #-------------------------------------------------------------------------------
    def render_annotate(self, fig, s, xy, **kwargs):
    #-------------------------------------------------------------------------------
        '''
        plot a trend line
        
        see http://matplotlib.org/api/axes_api.html#matplotlib.axes.Axes.annotate for parameters
        '''
        ax = fig.get_axes()[0]  # only one axes instance
        ax.annotate(s,xy,**kwargs)
        
    #-------------------------------------------------------------------------------
    def render_trendline(self, fig, label, thesestats=None, color=None):
    #-------------------------------------------------------------------------------
        '''
        plot a trend line
        
        :param label: label for trendline
        :param thesestats: list of :class:`AgeGradeStat`, or None if all stats to be used
        :param color: color per matplotlib for trendline, or None to automate
        :rtype: :class:`TrendLine` containing parameters of trendline
        '''
        
        ax = fig.get_axes()[0]  # only one axes instance

        if not thesestats:
            thesestats = self.stats
        
        x = [mdates.date2num(s.date) for s in thesestats]
        y = [s.ag for s in thesestats]
        
        slope,intercept,rvalue,pvalue,stderr = stats.linregress(x,y)
        xline = ax.get_xlim()  # returns floats, not datetimes
        yline = [slope*thisx+intercept for thisx in xline]
        
        if color:
            ax.plot(xline,yline,color=color,linestyle='-',label=label)
        else:
            ax.plot(xline,yline,linestyle='-',label=label)

        return TrendLine(slope,intercept,rvalue,pvalue,stderr)
        
    #-------------------------------------------------------------------------------
    def save(self,fig):
    #-------------------------------------------------------------------------------
        '''
        save the plot in indicated file
        '''
        outfile = self.get_outfilename()
    
        ax = fig.get_axes()[0]  # only one axes instance
        smallfont = fontmgr.FontProperties(size='x-small')
        ax.legend(loc=1,bbox_to_anchor=(1.19, 1),prop=smallfont)    #bbox_to_anchor moves legend outside axes
        fig.savefig(outfile,format='png')
        
#-------------------------------------------------------------------------------
def main():
#-------------------------------------------------------------------------------

    usage = "     where:"
    usage += "\n        agfile is csv file containing Date, Distance (miles), AG headers"
    usage += "\n        who is name for chart header (default Lou)"

    parser = argparse.ArgumentParser(version='running {0}'.format(version.__version__))
    parser.add_argument('--agfile', help="age grade csv file, with fields 'Date', 'Distance (miles)', 'AG' (optional, takes precedence)", default=None)
    parser.add_argument('--ra', action='store_true', help="use --ra to get data from RunningAHEAD")
    parser.add_argument('--athlinks', action='store_true', help="use --athlinks to get data from athlinks [TBA]")
    parser.add_argument('-y', '--ylim', help="y limits, of the form (bottom,top), e.g., (55,80)")
    parser.add_argument('-w', '--who', help="specify name to be used in plot header, and to pick user for --ra and --athlinks")
    parser.add_argument('-b', '--dob', help="specify birth date for age grade, in mm/dd/yyyy format, required if --agfile or --athlinks specified", default=None)
    parser.add_argument('-g', '--gender', help="specify gender for age grade, M or F, required if --agfile or --athlinks specified", default=None)
    parser.add_argument('-s', '--size', action='store_true', help="use --size if circle size by distance is desired")
    args = parser.parse_args()
    
    # get arguments
    agfile = args.agfile
    usera = args.ra
    useathlinks = args.athlinks
    who = args.who
    if args.dob:
        dt_dob = tdisp.asc2dt(args.dob)
    else:
        dt_dob = None
    gender = args.gender
    
    size = args.size
    if args.ylim:
        try:
            ylim = eval(args.ylim)
            if len(ylim) != 2 or type(ylim[0]) not in [int,float] or type(ylim[0]) not in [int,float]:
                raise ValueError
        except:
            print("YLIM argument must be of the form(bottom,top), e.g., (55,80)")
            return
    else:
        ylim = None

    # set up to analyze age grade
    aag = AnalyzeAgeGrade(size)
    aag.set_runner(who,gender,dt_dob)
    
    # get data from age grade csv file
    if agfile:
        aag.getdatafromfile(agfile)
        
    # get data from RunningAHEAD
    if usera:
        aag.getdatafromra()
    
    # TODO: combine file data with RA data -- currently RA data takes precedence
    
    # add age grade percentage to each result
    aag.crunch()
    
    # set up rendering parameters in aag
    # TODO: add option to control color map setup
    aag.set_colormap()
    if ylim:
        aag.set_ylim(ylim[0],ylim[1])
    
    # plot statistics
    fig = plt.figure()
    ax = fig.add_subplot(111)
    aag.render_stats(fig)
    aag.render_trendline(fig,'trend',color='k')
    aag.save(fig)
    
    
# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




#!/usr/bin/python
# ##########################################################################################
#	xmldict.py
#		turn xml into dictionary
#		based on https://systemausfall.org/websvn/codekasten/xml2typo3/xmlreader.py?op=file&rev=220&sc=0
#
#	Date		Author		Reason
#	----		------		------
#	10/25/09	Lou King	Create
#
# ##########################################################################################

# standard
import xml.dom.minidom
import pdb

class NotTextNodeError:
    pass


def getTextFromNode(node):
    """
    scans through all children of node and gathers the
    text. if node has non-text child-nodes, then
    NotTextNodeError is raised.
    """
    t = ""
    for n in node.childNodes:
        if n.nodeType == n.TEXT_NODE:
            t += n.nodeValue
        else:
            raise NotTextNodeError
    return t


def nodeToDic(node):
    """
    nodeToDic() scans through the children of node and makes a
    dictionary from the content.
    three cases are differentiated:
    - if the node contains no other nodes, it is a text-node
    and {nodeName:text} is merged into the dictionary.
    - if there is more than one child with the same name
    then these children will be appended to a list and this
    list is merged to the dictionary in the form: {nodeName:list}.
    - else, nodeToDic() will call itself recursively on
    the nodes children (merging {nodeName:nodeToDic()} to
    the dictionary).
    """
    dic = {} 
    multlist = {} # holds temporary lists where there are multiple children
    sibs = []
    for n in node.childNodes:
        sibs.append (n.nodeName)
    for n in node.childNodes:
        multiple = False 
        if n.nodeType != n.ELEMENT_NODE:
            continue
        # find out if there are multiple records    
        #if len(node.getElementsByTagName(n.nodeName)) > 1:
		# check if there are siblings with the same name
        if sibs.count(n.nodeName) > 1:
            multiple = True 
            # and set up the list to hold the values
            if n.nodeName not in multlist:
                multlist[n.nodeName] = []
        
        try:
            #text node
            text = getTextFromNode(n).strip().encode('utf-8')
        except NotTextNodeError:
            if multiple:
                # append to our list
                multlist[n.nodeName].append(nodeToDic(n))
                dic.update({n.nodeName:multlist[n.nodeName]})
                continue
            else: 
                # 'normal' node
                dic.update({n.nodeName:nodeToDic(n)})
                continue

        # text node
        if multiple:
            multlist[n.nodeName].append(text)
            dic.update({n.nodeName:multlist[n.nodeName]})
        else:
            dic.update({n.nodeName:text})
    return dic


def readXmlFile(filename):
    dom = xml.dom.minidom.parse(filename)
    return nodeToDic(dom)

def readXmlString(xmlstring):
    dom = xml.dom.minidom.parseString(xmlstring)
    return nodeToDic(dom)




if __name__ == "__main__":
	dic = readXmlFile("history.tcx")
	pdb.set_trace()




#!/usr/bin/python
###########################################################################################
#   racewx-render - render data from racewx.py
#
#	Date		Author		Reason
#	----		------		------
#	03/29/13        Lou King        Create
#
###########################################################################################
'''
racewx-render - render data from racewx.py
===================================================
'''

# standard
import argparse
import csv
import copy
import re
import math

# pypi
import pytz 

# github

# other
import matplotlib.pyplot as plt
from matplotlib import dates
import matplotlib.font_manager as fm

# home grown
from running.running import racewx, version
from loutilities import timeu

#----------------------------------------------------------------------
def main():
#----------------------------------------------------------------------

    parser = argparse.ArgumentParser(version='{0} {1}'.format('running', version.__version__))
    parser.add_argument('infile',help='file generated by racewx')
    parser.add_argument('racename',help='race name')
    args = parser.parse_args()
    
    infile = args.infile
    racename = args.racename

    # get input
    _WX = open(infile,'r',newline='')
    WX = csv.DictReader(_WX)
    wxdata = []
    for wx in WX:
        wxdata.append(wx)
    _WX.close()
    
    # for now, filter out all but the max 'exectime' entries
    lastexec = max([int(wx['exectime']) for wx in wxdata])
    while int(wxdata[0]['exectime']) != lastexec:
        wxdata.pop(0)
        
    # pull out fields to plot
    wxplot = {}
    plotfields = ['time','temperature','windchill','heatindex','dewpoint','windSpeed','windBearing','cloudCover','precipProbability','precipIntensity','cloudCover']
    for f in plotfields:
        wxplot[f] = [float(wx[f]) if wx[f]!='' else None for wx in wxdata]

    # get range on 30 minute boundaries
    starttime = int(wxplot['time'][0])
    fintime   = int(wxplot['time'][-1])
    adjstart  = (starttime // (30*60)) * (30*60)     # rounds to next lowest 30 minute boundary
    adjfin    = ((fintime-1 + 30*60) // (30*60)) * (30*60)         # rounds to next highest 30 minute boundary
    startdt = timeu.epoch2dt(adjstart)
    findt   = timeu.epoch2dt(adjfin)
    
    # time zone stuff, based on starting point
    lat = float(wxdata[0]['lat'])
    lon = float(wxdata[0]['lon'])
    tzid = racewx.gettzid(lat, lon)
    tz = pytz.timezone(tzid)
    wxplot['localtime'] = [timeu.utcdt2tzdt(timeu.epoch2dt(tm),tzid) for tm in wxplot['time']]
    
    # plot data
    fig = plt.figure()
    ttitle = timeu.asctime('%m/%d/%Y')
    racedate = ttitle.epoch2asc(wxplot['time'][0])
    fdate = ttitle.epoch2asc(lastexec)
    fig.suptitle('forecast for {race} {date}\nforecast date {fdate}\nPowered by Forecast.io'.format(race=racename,date=racedate,fdate=fdate),size='small')
    
    # set some formatting parameters
    lw = 0.5    # line width
    windcolor = 'b'
    legendx = 1.35
    
    # plot control
    exists = {}
    for f in ['windchill','heatindex']:
        exists[f] = len([it for it in wxplot[f] if it is not None]) != 0
    for f in ['precipIntensity']:
        exists[f] = len([it for it in wxplot[f] if it > 0.0]) != 0

    # plot temperatures
    ax1 = fig.add_subplot(311)
    ax1.plot(wxplot['localtime'],wxplot['temperature'],'k-',label='temperature', linewidth=lw)
    if exists['windchill']:
        ax1.plot(wxplot['localtime'],wxplot['windchill'],'b-',label='wind chill', linewidth=lw)
    if exists['heatindex']:
        ax1.plot(wxplot['localtime'],wxplot['heatindex'],'r-',label='heat index', linewidth=lw)
    ax1.plot(wxplot['localtime'],wxplot['dewpoint'],'g-',label='dew point', linewidth=lw)
    
    ax1.set_xlim(startdt,findt)
    fig.subplots_adjust(top=0.88,right=0.75,bottom=0.15)
    
    hfmt = dates.DateFormatter('%H:%M',tz=tz)
    ax1.xaxis.set_major_formatter(hfmt)
    ax1.xaxis.set_major_locator(dates.MinuteLocator(interval=30))    
    ax1.grid('on')
    plt.setp(ax1.get_xticklabels(), visible=False)
    plt.setp(ax1.get_yticklabels(), fontsize='small')
    ax1.set_ylabel('degrees  \nFahrenheit', fontsize='small')

    #font = fm.FontProperties(fname='Humor-Sans.ttf')
    font = fm.FontProperties()
    xsmallfont = copy.deepcopy(font)
    xsmallfont.set_size('x-small')
    ax1.legend(prop=xsmallfont,loc='upper right', bbox_to_anchor=(legendx, 1))

    # plot wind
    ax2 = fig.add_subplot(312)
    ax2.plot(wxplot['localtime'],wxplot['windSpeed'],label='wind speed', linewidth=lw, color=windcolor)
    # note polar-> rectangular flips x,y from standard transformation because theta is from North instead of East
    # not sure why need to invert U and V to get barb to point in right direction.  Maybe vector comes from U,V and points to origin?
    U = [-1*wxplot['windSpeed'][i]*math.sin(math.radians(wxplot['windBearing'][i])) for i in range(len(wxplot['windSpeed']))] 
    V = [-1*wxplot['windSpeed'][i]*math.cos(math.radians(wxplot['windBearing'][i])) for i in range(len(wxplot['windSpeed']))]
    xdates = dates.date2num(wxplot['localtime'])    # barbs requires floats, not datetime
    ax2.barbs(xdates,wxplot['windSpeed'], U, V, length=5, barbcolor=windcolor, flagcolor=windcolor, linewidth=lw)

    ax2.set_xlim(dates.date2num(startdt),dates.date2num(findt))
    miny,maxy = ax2.get_ylim()
    ax2.set_ylim(round(miny*0.8),round(maxy*1.2))
    ax2.xaxis.set_major_formatter(hfmt)
    ax2.xaxis.set_major_locator(dates.MinuteLocator(interval=30))
    ax2.grid('on')
    #plt.setp(ax2.get_xticklabels(), rotation='vertical', fontsize='small')
    plt.setp(ax2.get_xticklabels(), visible=False)
    plt.setp(ax2.get_yticklabels(), fontsize='small')    
    ax2.set_ylabel('miles per hour', fontsize='small')
    ax2.legend(prop=xsmallfont,loc='upper right', bbox_to_anchor=(legendx, 1))
    
    ax3 = fig.add_subplot(313)
    precipprob = [100*(prob or 0) for prob in wxplot['precipProbability']]
    cloudcover = [100*(cover or 0) for cover in wxplot['cloudCover']]
    ax3.plot(wxplot['localtime'],precipprob,label='rain probability', linewidth=lw, color='b')
    ax3.plot(wxplot['localtime'],cloudcover,label='cloud cover', linewidth=lw, color='g')
    ax3.set_ylabel('percent', fontsize='small')

    ax3.set_xlim(dates.date2num(startdt),dates.date2num(findt))
    ax3.xaxis.set_major_formatter(hfmt)
    ax3.xaxis.set_major_locator(dates.MinuteLocator(interval=30))
    ax3.grid('on')
    ax3.set_ylim(0,100)
    plt.setp(ax3.get_xticklabels(), rotation='vertical', fontsize='small')
    plt.setp(ax3.get_yticklabels(), fontsize='small')    
    ax3.legend(prop=xsmallfont,loc='upper right', bbox_to_anchor=(legendx, 1.1))

    if exists['precipIntensity']:
        ax4 = ax3.twinx()
        #ax4.plot(wxplot['localtime'],wxplot['precipIntensity'],label='intensity', linewidth=lw, color='r')
        #ax4.set_yscale('log')
        ax4.semilogy(wxplot['localtime'],wxplot['precipIntensity'],label='intensity', nonposy='mask',linewidth=lw, color='r')
        #ax4.set_ylabel('precipitation 0.002 very light sprinkling, 0.017 light precipitation, 0.1 precipitation, and 0.4 very heavy precipitation')
        ax4.set_ylabel('intensity',fontsize='small')
        ax4.set_ylim(0,0.5)
        plt.setp(ax4.get_yticklabels(), fontsize='small')    
        ax4.legend(prop=xsmallfont,loc='upper right', bbox_to_anchor=(legendx, 0.75))

    tfile = timeu.asctime('%Y-%m-%d')
    fdate = tfile.epoch2asc(lastexec)
    racename = re.sub('\s','',racename) # remove whitespace
    outfile = 'race-weather-{race}-{fdate}.png'.format(race=racename,fdate=fdate)
    fig.savefig(outfile,format='png')


# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




# standard
from argparse import ArgumentParser
from csv import DictWriter
from datetime import timedelta

# pypi
from loutilities.xmldict import ConvertXmlToDict

class ParameterError(Exception): pass

def dist2miles(distel):
    dist = float(distel['_text'])
    distunits = distel['unit']
    if distunits == 'mi':
        pass
    elif distunits == 'km':
        dist /= 1.609344
    elif distunits == 'm':
        dist /= 1609.344
    else:
        raise ParameterError(f'unexpected distance unit {distunits}')
    
    return dist

def convertsecs(dur):
    return str(timedelta(seconds = float(dur)))
    
def main():
    parser = ArgumentParser()
    parser.add_argument('-X', '--xmlfile', help='input xml file, from RunningAHEAD export', required=True)
    parser.add_argument('-C', '--csvfile', help='output csv file', required=True)
    args = parser.parse_args()

    fieldnames = 'date,time,type,subtype,dist,duration,equipment,route,temp,notes'.split(',')
    workouts = ConvertXmlToDict(args.xmlfile)
    with open(args.csvfile, 'w', newline='') as oscsvfile:
        csvfile = DictWriter(oscsvfile, fieldnames=fieldnames)
        csvfile.writeheader()
        for wo in workouts['RunningAHEADLog']['EventCollection']['Event']:
            if wo['typeName'] not in ['Run', 'Bike', 'Walk']: continue
            datetime = wo['time']
            datetimesplit = datetime.split('T')
            date = datetimesplit[0]
            time = datetimesplit[1] if len(datetimesplit) == 2 else ''
            time = time[:-1] if time and time[-1] == 'Z' else time
            row = {
                'date':      date,
                'time':      time,
                'type':      wo['typeName'],
                'subtype':   wo['subtypeName'] if 'subtypeName' in wo else '',
                'dist':      dist2miles(wo['Distance']) if 'Distance' in wo else '',
                'duration':  convertsecs(wo['Duration']['seconds']) if 'Duration' in wo else '',
                'equipment': wo['Equipment']['_text'] if 'Equipment' in wo else '',
                'route':     wo['Route']['_text'] if 'Route' in wo else '',
                'temp':      wo['EnvironmentalConditions']['Temperature']['_text'] if 'EnvironmentalConditions' in wo and 'Temperature' in wo['EnvironmentalConditions'] else '',
                'notes':     wo['Notes'] if 'Notes' in wo else '',
            }
            csvfile.writerow(row)
    
if __name__ == "__main__":
    main()


from xml.sax.saxutils import XMLFilterBase

class text_normalize_filter(XMLFilterBase):
    """
    SAX filter to ensure that contiguous white space nodes are
    delivered merged into a single node
    """
    
    def __init__(self, upstream, downstream):
        XMLFilterBase.__init__(self, upstream)
        self._downstream = downstream
        self._accumulator = []
        return

    def _complete_text_node(self):
        if self._accumulator:
            self._downstream.characters(''.join(self._accumulator))
            self._accumulator = []
        return

    def startElement(self, name, attrs):
        self._complete_text_node()
        self._downstream.startElement(name, attrs)
        return

    def startElementNS(self, name, qname, attrs):
        self._complete_text_node()
        self._downstream.startElementNS(name, qname, attrs)
        return

    def endElement(self, name):
        self._complete_text_node()
        self._downstream.endElement(name)
        return

    def endElementNS(self, name, qname):
        self._complete_text_node()
        self._downstream.endElementNS(name, qname)
        return

    def processingInstruction(self, target, body):
        self._complete_text_node()
        self._downstream.processingInstruction(target, body)
        return

    def comment(self, body):
        self._complete_text_node()
        self._downstream.comment(body)
        return

    def characters(self, text):
        self._accumulator.append(text)
        return

    def ignorableWhitespace(self, ws):
        self._accumulator.append(text)
        return


if __name__ == "__main__":
    import sys
    from xml import sax
    from xml.sax.saxutils import XMLGenerator
    parser = sax.make_parser()
    #XMLGenerator is a special SAX handler that merely writes
    #SAX events back into an XML document
    downstream_handler = XMLGenerator()
    #upstream, the parser, downstream, the next handler in the chain
    filter_handler = text_normalize_filter(parser, downstream_handler)
    #The SAX filter base is designed so that the filter takes
    #on much of the interface of the parser itself, including the
    #"parse" method
    filter_handler.parse(sys.argv[1])